@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: debug.h 11151 2012-04-24 11:01:59Z sshwarts $
+// $Id: debug.h 11580 2013-01-19 20:45:03Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2011  The Bochs Project
+//  Copyright (C) 2001-2013  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -219,6 +219,7 @@ void bx_dbg_set_reg8h_value(unsigned reg, Bit8u value);
 void bx_dbg_set_reg16_value(unsigned reg, Bit16u value);
 void bx_dbg_set_reg32_value(unsigned reg, Bit32u value);
 void bx_dbg_set_reg64_value(unsigned reg, Bit64u value);
+void bx_dbg_set_rip_value(bx_address value);
 void bx_dbg_load_segreg(unsigned reg, unsigned value);
 bx_address bx_dbg_get_laddr(Bit16u sel, bx_address ofs);
 void bx_dbg_step_over_command(void);
@@ -235,7 +236,8 @@ void bx_dbg_modebp_command(void);
 void bx_dbg_vmexitbp_command(void);
 void bx_dbg_where_command(void);
 void bx_dbg_print_string_command(bx_address addr);
-void bx_dbg_xlate_address(bx_lin_address address);
+void bx_dbg_xlate_address(bx_lin_address laddr);
+void bx_dbg_tlb_lookup(bx_lin_address laddr);
 void bx_dbg_show_command(const char*);
 void bx_dbg_print_stack_command(unsigned nwords);
 void bx_dbg_print_watchpoints(void);
@@ -314,7 +316,7 @@ void bx_dbg_restore_command(const char *param_name, const char *path);
 void bx_dbg_show_param_command(const char *param);
 
 int bx_dbg_show_symbolic(void);
-void bx_dbg_set_symbol_command(const char *symbol, Bit32u val);
+void bx_dbg_set_symbol_command(const char *symbol, bx_address val);
 const char* bx_dbg_symbolic_address(Bit32u context, Bit32u eip, Bit32u base);
 const char* bx_dbg_symbolic_address_16bit(Bit32u eip, Bit32u cs);
 int bx_dbg_symbol_command(const char* filename, bx_bool global, Bit32u offset);
@@ -345,19 +347,6 @@ typedef enum {
   BREAK_POINT_MAGIC, BREAK_POINT_READ, BREAK_POINT_WRITE, BREAK_POINT_TIME
 } break_point_t;
 
-#define BX_DBG_REG_EIP          10
-#define BX_DBG_REG_EFLAGS       11
-#define BX_DBG_REG_CS           20
-#define BX_DBG_REG_SS           21
-#define BX_DBG_REG_DS           22
-#define BX_DBG_REG_ES           23
-#define BX_DBG_REG_FS           24
-#define BX_DBG_REG_GS           25
-#define BX_DBG_REG_CR0          30
-#define BX_DBG_REG_CR2          32
-#define BX_DBG_REG_CR3          33
-#define BX_DBG_REG_CR4          34
-
 #define BX_DBG_PENDING_DMA 1
 #define BX_DBG_PENDING_IRQ 2
 
