@@ -95,51 +95,52 @@
      BX_TOKEN_TAKE = 313,
      BX_TOKEN_DMA = 314,
      BX_TOKEN_IRQ = 315,
-     BX_TOKEN_HEX = 316,
-     BX_TOKEN_DISASM = 317,
-     BX_TOKEN_INSTRUMENT = 318,
-     BX_TOKEN_STRING = 319,
-     BX_TOKEN_STOP = 320,
-     BX_TOKEN_DOIT = 321,
-     BX_TOKEN_CRC = 322,
-     BX_TOKEN_TRACE = 323,
-     BX_TOKEN_TRACEREG = 324,
-     BX_TOKEN_TRACEMEM = 325,
-     BX_TOKEN_SWITCH_MODE = 326,
-     BX_TOKEN_SIZE = 327,
-     BX_TOKEN_PTIME = 328,
-     BX_TOKEN_TIMEBP_ABSOLUTE = 329,
-     BX_TOKEN_TIMEBP = 330,
-     BX_TOKEN_MODEBP = 331,
-     BX_TOKEN_VMEXITBP = 332,
-     BX_TOKEN_PRINT_STACK = 333,
-     BX_TOKEN_WATCH = 334,
-     BX_TOKEN_UNWATCH = 335,
-     BX_TOKEN_READ = 336,
-     BX_TOKEN_WRITE = 337,
-     BX_TOKEN_SHOW = 338,
-     BX_TOKEN_LOAD_SYMBOLS = 339,
-     BX_TOKEN_SYMBOLS = 340,
-     BX_TOKEN_LIST_SYMBOLS = 341,
-     BX_TOKEN_GLOBAL = 342,
-     BX_TOKEN_WHERE = 343,
-     BX_TOKEN_PRINT_STRING = 344,
-     BX_TOKEN_NUMERIC = 345,
-     BX_TOKEN_PAGE = 346,
-     BX_TOKEN_HELP = 347,
-     BX_TOKEN_CALC = 348,
-     BX_TOKEN_VGA = 349,
-     BX_TOKEN_DEVICE = 350,
-     BX_TOKEN_COMMAND = 351,
-     BX_TOKEN_GENERIC = 352,
-     BX_TOKEN_RSHIFT = 353,
-     BX_TOKEN_LSHIFT = 354,
-     BX_TOKEN_REG_IP = 355,
-     BX_TOKEN_REG_EIP = 356,
-     BX_TOKEN_REG_RIP = 357,
-     INDIRECT = 358,
-     NEG = 359,
-     NOT = 360
+     BX_TOKEN_TLB = 316,
+     BX_TOKEN_HEX = 317,
+     BX_TOKEN_DISASM = 318,
+     BX_TOKEN_INSTRUMENT = 319,
+     BX_TOKEN_STRING = 320,
+     BX_TOKEN_STOP = 321,
+     BX_TOKEN_DOIT = 322,
+     BX_TOKEN_CRC = 323,
+     BX_TOKEN_TRACE = 324,
+     BX_TOKEN_TRACEREG = 325,
+     BX_TOKEN_TRACEMEM = 326,
+     BX_TOKEN_SWITCH_MODE = 327,
+     BX_TOKEN_SIZE = 328,
+     BX_TOKEN_PTIME = 329,
+     BX_TOKEN_TIMEBP_ABSOLUTE = 330,
+     BX_TOKEN_TIMEBP = 331,
+     BX_TOKEN_MODEBP = 332,
+     BX_TOKEN_VMEXITBP = 333,
+     BX_TOKEN_PRINT_STACK = 334,
+     BX_TOKEN_WATCH = 335,
+     BX_TOKEN_UNWATCH = 336,
+     BX_TOKEN_READ = 337,
+     BX_TOKEN_WRITE = 338,
+     BX_TOKEN_SHOW = 339,
+     BX_TOKEN_LOAD_SYMBOLS = 340,
+     BX_TOKEN_SYMBOLS = 341,
+     BX_TOKEN_LIST_SYMBOLS = 342,
+     BX_TOKEN_GLOBAL = 343,
+     BX_TOKEN_WHERE = 344,
+     BX_TOKEN_PRINT_STRING = 345,
+     BX_TOKEN_NUMERIC = 346,
+     BX_TOKEN_PAGE = 347,
+     BX_TOKEN_HELP = 348,
+     BX_TOKEN_CALC = 349,
+     BX_TOKEN_VGA = 350,
+     BX_TOKEN_DEVICE = 351,
+     BX_TOKEN_COMMAND = 352,
+     BX_TOKEN_GENERIC = 353,
+     BX_TOKEN_RSHIFT = 354,
+     BX_TOKEN_LSHIFT = 355,
+     BX_TOKEN_REG_IP = 356,
+     BX_TOKEN_REG_EIP = 357,
+     BX_TOKEN_REG_RIP = 358,
+     INDIRECT = 359,
+     NEG = 360,
+     NOT = 361
    };
 #endif
 /* Tokens.  */
@@ -201,51 +202,52 @@
 #define BX_TOKEN_TAKE 313
 #define BX_TOKEN_DMA 314
 #define BX_TOKEN_IRQ 315
-#define BX_TOKEN_HEX 316
-#define BX_TOKEN_DISASM 317
-#define BX_TOKEN_INSTRUMENT 318
-#define BX_TOKEN_STRING 319
-#define BX_TOKEN_STOP 320
-#define BX_TOKEN_DOIT 321
-#define BX_TOKEN_CRC 322
-#define BX_TOKEN_TRACE 323
-#define BX_TOKEN_TRACEREG 324
-#define BX_TOKEN_TRACEMEM 325
-#define BX_TOKEN_SWITCH_MODE 326
-#define BX_TOKEN_SIZE 327
-#define BX_TOKEN_PTIME 328
-#define BX_TOKEN_TIMEBP_ABSOLUTE 329
-#define BX_TOKEN_TIMEBP 330
-#define BX_TOKEN_MODEBP 331
-#define BX_TOKEN_VMEXITBP 332
-#define BX_TOKEN_PRINT_STACK 333
-#define BX_TOKEN_WATCH 334
-#define BX_TOKEN_UNWATCH 335
-#define BX_TOKEN_READ 336
-#define BX_TOKEN_WRITE 337
-#define BX_TOKEN_SHOW 338
-#define BX_TOKEN_LOAD_SYMBOLS 339
-#define BX_TOKEN_SYMBOLS 340
-#define BX_TOKEN_LIST_SYMBOLS 341
-#define BX_TOKEN_GLOBAL 342
-#define BX_TOKEN_WHERE 343
-#define BX_TOKEN_PRINT_STRING 344
-#define BX_TOKEN_NUMERIC 345
-#define BX_TOKEN_PAGE 346
-#define BX_TOKEN_HELP 347
-#define BX_TOKEN_CALC 348
-#define BX_TOKEN_VGA 349
-#define BX_TOKEN_DEVICE 350
-#define BX_TOKEN_COMMAND 351
-#define BX_TOKEN_GENERIC 352
-#define BX_TOKEN_RSHIFT 353
-#define BX_TOKEN_LSHIFT 354
-#define BX_TOKEN_REG_IP 355
-#define BX_TOKEN_REG_EIP 356
-#define BX_TOKEN_REG_RIP 357
-#define INDIRECT 358
-#define NEG 359
-#define NOT 360
+#define BX_TOKEN_TLB 316
+#define BX_TOKEN_HEX 317
+#define BX_TOKEN_DISASM 318
+#define BX_TOKEN_INSTRUMENT 319
+#define BX_TOKEN_STRING 320
+#define BX_TOKEN_STOP 321
+#define BX_TOKEN_DOIT 322
+#define BX_TOKEN_CRC 323
+#define BX_TOKEN_TRACE 324
+#define BX_TOKEN_TRACEREG 325
+#define BX_TOKEN_TRACEMEM 326
+#define BX_TOKEN_SWITCH_MODE 327
+#define BX_TOKEN_SIZE 328
+#define BX_TOKEN_PTIME 329
+#define BX_TOKEN_TIMEBP_ABSOLUTE 330
+#define BX_TOKEN_TIMEBP 331
+#define BX_TOKEN_MODEBP 332
+#define BX_TOKEN_VMEXITBP 333
+#define BX_TOKEN_PRINT_STACK 334
+#define BX_TOKEN_WATCH 335
+#define BX_TOKEN_UNWATCH 336
+#define BX_TOKEN_READ 337
+#define BX_TOKEN_WRITE 338
+#define BX_TOKEN_SHOW 339
+#define BX_TOKEN_LOAD_SYMBOLS 340
+#define BX_TOKEN_SYMBOLS 341
+#define BX_TOKEN_LIST_SYMBOLS 342
+#define BX_TOKEN_GLOBAL 343
+#define BX_TOKEN_WHERE 344
+#define BX_TOKEN_PRINT_STRING 345
+#define BX_TOKEN_NUMERIC 346
+#define BX_TOKEN_PAGE 347
+#define BX_TOKEN_HELP 348
+#define BX_TOKEN_CALC 349
+#define BX_TOKEN_VGA 350
+#define BX_TOKEN_DEVICE 351
+#define BX_TOKEN_COMMAND 352
+#define BX_TOKEN_GENERIC 353
+#define BX_TOKEN_RSHIFT 354
+#define BX_TOKEN_LSHIFT 355
+#define BX_TOKEN_REG_IP 356
+#define BX_TOKEN_REG_EIP 357
+#define BX_TOKEN_REG_RIP 358
+#define INDIRECT 359
+#define NEG 360
+#define NOT 361
 
 
 
@@ -264,7 +266,7 @@ typedef union YYSTYPE
 
 
 /* Line 2068 of yacc.c  */
-#line 268 "y.tab.h"
+#line 270 "y.tab.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
