@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: corei7_ivy_bridge_3770K.cc 11217 2012-06-14 18:56:47Z sshwarts $
+// $Id: corei7_ivy_bridge_3770K.cc 11687 2013-05-20 18:15:35Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2012 Stanislav Shwartsman
@@ -44,7 +44,7 @@ corei7_ivy_bridge_3770k_t::corei7_ivy_bridge_3770k_t(BX_CPU_C *cpu): bx_cpuid_t(
   if (! BX_SUPPORT_MONITOR_MWAIT)
     BX_INFO(("WARNING: MONITOR/MWAIT support is not compiled in !"));
 
-  BX_INFO(("WARNING: RDRAND is not implemented yet !"));
+  BX_INFO(("WARNING: RDRAND would not produce true random numbers !"));
 }
 
 void corei7_ivy_bridge_3770k_t::get_cpuid_leaf(Bit32u function, Bit32u subfunction, cpuid_function_t *leaf) const
@@ -151,6 +151,7 @@ Bit64u corei7_ivy_bridge_3770k_t::get_isa_extensions_bitmask(void) const
          BX_ISA_FSGSBASE |
          BX_ISA_AVX |
          BX_ISA_AVX_F16C |
+         BX_ISA_RDRAND |
          BX_ISA_CMPXCHG16B |
          BX_ISA_LM_LAHF_SAHF;
 }
@@ -166,7 +167,7 @@ Bit32u corei7_ivy_bridge_3770k_t::get_cpu_extensions_bitmask(void) const
          BX_CPU_MTRR |
          BX_CPU_PAT |
          BX_CPU_XAPIC |
-      /* BX_CPU_X2APIC | */
+         BX_CPU_X2APIC |
          BX_CPU_LONG_MODE |
          BX_CPU_NX |
          BX_CPU_PCID |
@@ -290,7 +291,7 @@ void corei7_ivy_bridge_3770k_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) con
   //   [18:18] DCA - Direct Cache Access
   // * [19:19] SSE4.1 Instructions
   // * [20:20] SSE4.2 Instructions
-  //   [21:21] X2APIC
+  // * [21:21] X2APIC
   //   [22:22] MOVBE instruction
   // * [23:23] POPCNT instruction
   // * [24:24] TSC Deadline
@@ -321,14 +322,14 @@ void corei7_ivy_bridge_3770k_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) con
               BX_CPUID_EXT_PCID |
               BX_CPUID_EXT_SSE4_1 |
               BX_CPUID_EXT_SSE4_2 |
-           /* BX_CPUID_EXT_X2APIC | */
+              BX_CPUID_EXT_X2APIC |
               BX_CPUID_EXT_POPCNT |
               BX_CPUID_EXT_TSC_DEADLINE |
               BX_CPUID_EXT_AES |
               BX_CPUID_EXT_XSAVE |
               BX_CPUID_EXT_AVX |
-              BX_CPUID_EXT_AVX_F16C;
-           // BX_CPUID_EXT_RDRAND not implemented yet
+              BX_CPUID_EXT_AVX_F16C |
+              BX_CPUID_EXT_RDRAND;
   if (cpu->cr4.get_OSXSAVE())
     leaf->ecx |= BX_CPUID_EXT_OSXSAVE;
 
@@ -554,10 +555,16 @@ void corei7_ivy_bridge_3770k_t::get_std_cpuid_leaf_7(Bit32u subfunction, cpuid_f
 void corei7_ivy_bridge_3770k_t::get_std_cpuid_leaf_A(cpuid_function_t *leaf) const
 {
   // CPUID function 0x0000000A - Architectural Performance Monitoring Leaf
+/*
   leaf->eax = 0x07300403;
   leaf->ebx = 0x00000000;
   leaf->ecx = 0x00000000;
   leaf->edx = 0x00000603;
+*/
+  leaf->eax = 0; // reporting true capabilities breaks Win7 x64 installation
+  leaf->ebx = 0;
+  leaf->ecx = 0;
+  leaf->edx = 0;
 
   BX_INFO(("WARNING: Architectural Performance Monitoring is not implemented"));
 }
