@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: corei7_sandy_bridge_2600K.cc 11217 2012-06-14 18:56:47Z sshwarts $
+// $Id: corei7_sandy_bridge_2600K.cc 11685 2013-05-07 15:34:58Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2011-2012 Stanislav Shwartsman
@@ -514,10 +514,16 @@ void corei7_sandy_bridge_2600k_t::get_std_cpuid_leaf_6(cpuid_function_t *leaf) c
 void corei7_sandy_bridge_2600k_t::get_std_cpuid_leaf_A(cpuid_function_t *leaf) const
 {
   // CPUID function 0x0000000A - Architectural Performance Monitoring Leaf
+/*
   leaf->eax = 0x07300803;
   leaf->ebx = 0x00000000;
   leaf->ecx = 0x00000000;
   leaf->edx = 0x00000603;
+*/
+  leaf->eax = 0; // reporting true capabilities breaks Win7 x64 installation
+  leaf->ebx = 0;
+  leaf->ecx = 0;
+  leaf->edx = 0;
 
   BX_INFO(("WARNING: Architectural Performance Monitoring is not implemented"));
 }
