@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: generic_cpuid.cc 11276 2012-07-12 14:51:54Z sshwarts $
+// $Id: generic_cpuid.cc 11688 2013-05-20 18:18:52Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2011-2012 Stanislav Shwartsman
@@ -28,6 +28,16 @@
 
 #define LOG_THIS cpu->
 
+#undef BX_CPUID_SUPPORT_ISA_EXTENSION
+
+#define BX_CPUID_SUPPORT_ISA_EXTENSION(feature) \
+   (this->isa_extensions_bitmask & (feature))
+
+#undef BX_CPUID_SUPPORT_CPU_EXTENSION
+
+#define BX_CPUID_SUPPORT_CPU_EXTENSION(feature) \
+   (this->cpu_extensions_bitmask & (feature))
+
 bx_cpuid_t::bx_cpuid_t(BX_CPU_C *_cpu): cpu(_cpu)
 {
 #if BX_SUPPORT_SMP
@@ -664,31 +674,42 @@ void bx_generic_cpuid_t::init_isa_extensions_bitmask(void)
 #endif
 
 #if BX_CPU_LEVEL >= 4
+  static unsigned cpu_level = SIM->get_param_num(BXPN_CPUID_LEVEL)->get();
+  if (cpu_level < 5) {
+    BX_PANIC(("Unsupported CPUID level selected %d", cpu_level));
+  }
+
   features_bitmask |= BX_ISA_486;
 
 #if BX_CPU_LEVEL >= 5
-  features_bitmask |= BX_ISA_PENTIUM;
-
   static bx_bool mmx_enabled = SIM->get_param_bool(BXPN_CPUID_MMX)->get();
-  if (mmx_enabled)
-    features_bitmask |= BX_ISA_MMX;
+
+  if (cpu_level >= 5) {
+    features_bitmask |= BX_ISA_PENTIUM;
+
+    if (mmx_enabled)
+      features_bitmask |= BX_ISA_MMX;
 
 #if BX_SUPPORT_3DNOW
-  features_bitmask |= BX_ISA_3DNOW;
-
-  if (! mmx_enabled) {
-    BX_PANIC(("PANIC: 3DNOW emulation requires MMX support !"));
-    return;
-  }
+    features_bitmask |= BX_ISA_3DNOW;
+    if (! mmx_enabled) {
+      BX_PANIC(("PANIC: 3DNOW emulation requires MMX support !"));
+      return;
+    }
 #endif
+  }
 
 #if BX_CPU_LEVEL >= 6
-  features_bitmask |= BX_ISA_P6;
+  if (cpu_level >= 6)
+    features_bitmask |= BX_ISA_P6;
 
 #if BX_SUPPORT_MONITOR_MWAIT
   static bx_bool mwait_enabled = SIM->get_param_bool(BXPN_CPUID_MWAIT)->get();
-  if (mwait_enabled)
+  if (mwait_enabled) {
     features_bitmask |= BX_ISA_MONITOR_MWAIT;
+    if (cpu_level < 6)
+      BX_PANIC(("PANIC: MONITOR/MWAIT emulation requires P6 CPU level support !"));
+  }
 #endif
 
   static unsigned sse_enabled = SIM->get_param_enum(BXPN_CPUID_SSE)->get();
@@ -712,7 +733,7 @@ void bx_generic_cpuid_t::init_isa_extensions_bitmask(void)
   };
 
   if (sse_enabled) {
-    if (mmx_enabled == 0 || BX_CPU_LEVEL < 6) {
+    if (mmx_enabled == 0 || cpu_level < 6 || BX_CPU_LEVEL < 6) {
       BX_PANIC(("PANIC: SSE support requires P6 emulation with MMX enabled !"));
       return;
     }
@@ -737,8 +758,11 @@ void bx_generic_cpuid_t::init_isa_extensions_bitmask(void)
   }
 
   static bx_bool sep_enabled = SIM->get_param_bool(BXPN_CPUID_SEP)->get();
-  if (sep_enabled)
+  if (sep_enabled) {
     features_bitmask |= BX_ISA_SYSENTER_SYSEXIT;
+    if (cpu_level < 6)
+      BX_PANIC(("PANIC: SYSENTER/SYSEXIT emulation requires P6 CPU level support !"));
+  }
 
   static bx_bool xsave_enabled = SIM->get_param_bool(BXPN_CPUID_XSAVE)->get();
   if (xsave_enabled) {
@@ -796,6 +820,9 @@ void bx_generic_cpuid_t::init_isa_extensions_bitmask(void)
 #if BX_SUPPORT_X86_64
   static bx_bool x86_64_enabled = SIM->get_param_bool(BXPN_CPUID_X86_64)->get();
   if (x86_64_enabled) {
+    if (cpu_level < 6)
+      BX_PANIC(("PANIC: x86-64 emulation requires P6 CPU level support !"));
+
     features_bitmask |= BX_ISA_CMPXCHG16B | BX_ISA_RDTSCP | BX_ISA_LM_LAHF_SAHF;
 
     if (sse_enabled < BX_CPUID_SUPPORT_SSE2) {
@@ -820,7 +847,7 @@ void bx_generic_cpuid_t::init_isa_extensions_bitmask(void)
   }
   else {
     if (BX_SUPPORT_VMX >= 2) {
-      BX_PANIC(("PANIC: VMXx2 emulation requires x86-64 support !"));
+      BX_PANIC(("PANIC: VMX=2 emulation requires x86-64 support !"));
       return;
     }
   }
@@ -954,9 +981,19 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
 {
   Bit32u features_bitmask = 0;
 
+  static unsigned cpu_level = SIM->get_param_num(BXPN_CPUID_LEVEL)->get();
+  if (cpu_level < 5) {
+    BX_PANIC(("Unsupported CPUID level selected %d", cpu_level));
+  }
+
 #if BX_SUPPORT_APIC
   static unsigned apic_enabled = SIM->get_param_enum(BXPN_CPUID_APIC)->get();
-  // determine SSE in runtime
+
+  if (cpu_level < 6) {
+    if (apic_enabled != BX_CPUID_SUPPORT_LEGACY_APIC && apic_enabled != BX_CPUID_SUPPORT_XAPIC)
+      BX_PANIC(("PANIC: APIC extensions emulation require P6 CPU level support !"));
+  }
+
   switch (apic_enabled) {
 #if BX_CPU_LEVEL >= 6
     case BX_CPUID_SUPPORT_X2APIC:
@@ -977,30 +1014,51 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
 #endif
 
 #if BX_CPU_LEVEL >= 5
-  features_bitmask |= BX_CPU_VME;
-  features_bitmask |= BX_CPU_DEBUG_EXTENSIONS;
-  features_bitmask |= BX_CPU_PSE;
+  if (cpu_level >= 5) {
+    features_bitmask |= BX_CPU_VME;
+    features_bitmask |= BX_CPU_DEBUG_EXTENSIONS;
+    features_bitmask |= BX_CPU_PSE;
+
+#if BX_PHY_ADDRESS_LONG
+    features_bitmask |= BX_CPU_PSE36;
+#endif
+  }
 
 #if BX_CPU_LEVEL >= 6
-  features_bitmask |= BX_CPU_PAE;
-  features_bitmask |= BX_CPU_PGE;
-#if BX_PHY_ADDRESS_LONG
-  features_bitmask |= BX_CPU_PSE36;
-#endif
-  features_bitmask |= BX_CPU_MTRR;
-  features_bitmask |= BX_CPU_PAT;
+  if (cpu_level >= 6) {
+    features_bitmask |= BX_CPU_PAE;
+    features_bitmask |= BX_CPU_PGE;
+    features_bitmask |= BX_CPU_MTRR;
+    features_bitmask |= BX_CPU_PAT;
+  }
 
-#if BX_SUPPORT_MISALIGNED_SSE
-  features_bitmask |= BX_CPU_MISALIGNED_SSE;
-#endif
+  static bx_bool misaligned_sse_enabled = SIM->get_param_bool(BXPN_CPUID_MISALIGNED_SSE)->get();
+  if (misaligned_sse_enabled) {
+    features_bitmask |= BX_CPU_MISALIGNED_SSE;
+    if (cpu_level < 6)
+      BX_PANIC(("PANIC: Misaligned SSE emulation requires P6 CPU level support !"));
+  }
 
   static bx_bool smep_enabled = SIM->get_param_bool(BXPN_CPUID_SMEP)->get();
-  if (smep_enabled)
+  if (smep_enabled) {
     features_bitmask |= BX_CPU_SMEP;
+    if (cpu_level < 6)
+      BX_PANIC(("PANIC: SMEP emulation requires P6 CPU level support !"));
+  }
+
+  static bx_bool smap_enabled = SIM->get_param_bool(BXPN_CPUID_SMAP)->get();
+  if (smap_enabled) {
+    features_bitmask |= BX_ISA_SMAP;
+    if (cpu_level < 6)
+      BX_PANIC(("PANIC: SMAP emulation requires P6 CPU level support !"));
+  }
 
 #if BX_SUPPORT_X86_64
   static bx_bool x86_64_enabled = SIM->get_param_bool(BXPN_CPUID_X86_64)->get();
   if (x86_64_enabled) {
+    if (cpu_level < 6)
+      BX_PANIC(("PANIC: x86-64 emulation requires P6 CPU level support !"));
+
     features_bitmask |= BX_CPU_LONG_MODE | BX_CPU_FFXSR | BX_CPU_NX;
 
     static bx_bool pcid_enabled = SIM->get_param_bool(BXPN_CPUID_PCID)->get();
@@ -1013,9 +1071,13 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
   }
 
 #if BX_SUPPORT_SVM
-  static unsigned svm_enabled = SIM->get_param_num(BXPN_CPUID_SVM)->get();
+  static bx_bool svm_enabled = SIM->get_param_bool(BXPN_CPUID_SVM)->get();
   if (svm_enabled) {
     features_bitmask |= BX_CPU_ALT_MOV_CR8 | BX_CPU_XAPIC_EXT; // auto-enable together with SVM
+    if (! x86_64_enabled) {
+      BX_PANIC(("PANIC: SVM emulation requires x86-64 support !"));
+      return;
+    }
   }
 #endif
 
@@ -1053,7 +1115,11 @@ void bx_generic_cpuid_t::init_vmx_extensions_bitmask(void)
                             BX_VMX_UNRESTRICTED_GUEST |
                             BX_VMX_DESCRIPTOR_TABLE_EXIT |
                             BX_VMX_X2APIC_VIRTUALIZATION |
-                            BX_VMX_PAUSE_LOOP_EXITING;
+                            BX_VMX_PAUSE_LOOP_EXITING |
+                            BX_VMX_EPT_ACCESS_DIRTY |
+                            BX_VMX_VINTR_DELIVERY |
+                            BX_VMX_VMCS_SHADOWING |
+                            BX_VMX_EPTP_SWITCHING | BX_VMX_EPT_EXCEPTION;
 
         features_bitmask |= BX_VMX_SAVE_DEBUGCTL_DISABLE |
                          /* BX_VMX_MONITOR_TRAP_FLAG | */ // not implemented yet
@@ -1095,12 +1161,13 @@ void bx_generic_cpuid_t::init_svm_extensions_bitmask(void)
 
 Bit32u bx_generic_cpuid_t::get_cpu_version_information(void) const
 {
+  static Bit32u level = SIM->get_param_num(BXPN_CPUID_LEVEL)->get();
   static Bit32u stepping = SIM->get_param_num(BXPN_CPUID_STEPPING)->get();
   static Bit32u model = SIM->get_param_num(BXPN_CPUID_MODEL)->get();
   static Bit32u family = SIM->get_param_num(BXPN_CPUID_FAMILY)->get();
 
-  if (family < 6 && family != BX_CPU_LEVEL)
-    BX_PANIC(("PANIC: CPUID family %x not matching configured cpu level %d", family, BX_CPU_LEVEL));
+  if (family < 6 && family != level)
+    BX_PANIC(("PANIC: CPUID family %x not matching configured cpu level %d", family, level));
 
   return ((family & 0xfff0) << 16) |
          ((model & 0xf0) << 12) |
@@ -1427,9 +1494,8 @@ Bit32u bx_generic_cpuid_t::get_ext2_cpuid_features(void) const
     features |= BX_CPUID_EXT2_LAHF_SAHF | BX_CPUID_EXT2_PREFETCHW;
 #endif
 
-#if BX_SUPPORT_MISALIGNED_SSE
-  features |= BX_CPUID_EXT2_MISALIGNED_SSE;
-#endif
+  if (BX_CPUID_SUPPORT_CPU_EXTENSION(BX_CPU_MISALIGNED_SSE))
+    features |= BX_CPUID_EXT2_MISALIGNED_SSE;
 
   if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_LZCNT))
     features |= BX_CPUID_EXT2_LZCNT;
@@ -1456,14 +1522,19 @@ Bit32u bx_generic_cpuid_t::get_ext3_cpuid_features(void) const
   //   [0:0]    FS/GS BASE access instructions
   //   [2:1]    reserved
   //   [3:3]    BMI1: Advanced Bit Manipulation Extensions
-  //   [4:4]    reserved
+  //   [4:4]    HLE: Hardware Lock Elision
   //   [5:5]    AVX2
   //   [6:6]    reserved
   //   [7:7]    SMEP: Supervisor Mode Execution Protection
   //   [8:8]    BMI2: Advanced Bit Manipulation Extensions
   //   [9:9]    Support for Enhanced REP MOVSB/STOSB
   //   [10:10]  Support for INVPCID instruction
-  //   [31:10]  reserved
+  //   [11:11]  RTM: Restricted Transactional Memory
+  //   [17:12]  reserved
+  //   [18:18]  RDSEED instruction support
+  //   [19:19]  ADCX/ADOX instructions support
+  //   [20:20]  SMAP: Supervisor Mode Access Prevention
+  //   [31:21]  reserved
   if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_FSGSBASE))
     features |= BX_CPUID_EXT3_FSGSBASE;
 
@@ -1482,6 +1553,12 @@ Bit32u bx_generic_cpuid_t::get_ext3_cpuid_features(void) const
   if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_INVPCID))
     features |= BX_CPUID_EXT3_INVPCID;
 
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_ADX))
+    features |= BX_CPUID_EXT3_ADX;
+
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_SMAP))
+    features |= BX_CPUID_EXT3_SMAP;
+
   return features;
 }
 
