@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: init.cc 11232 2012-06-24 17:52:45Z sshwarts $
+// $Id: init.cc 11674 2013-04-09 15:43:15Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2012  The Bochs Project
@@ -27,6 +27,8 @@
 
 #include "param_names.h"
 
+#include <stdlib.h>
+
 BX_CPU_C::BX_CPU_C(unsigned id): bx_cpuid(id)
 #if BX_CPU_LEVEL >= 4
    , cpuid(NULL)
@@ -51,110 +53,10 @@ BX_CPU_C::BX_CPU_C(unsigned id): bx_cpuid(id)
 #if BX_SUPPORT_SVM
   svm_extensions_bitmask = 0;
 #endif
+
+  srand(time(NULL)); // initialize random generator for RDRAND/RDSEED
 }
 
-#if BX_WITH_WX
-
-#define IF_SEG_REG_GET(x) \
-  if (!strcmp(param->get_name(), #x)) { \
-    return BX_CPU(cpu)->sregs[BX_SEG_REG_##x].selector.value; \
-  }
-#define IF_SEG_REG_SET(reg, val) \
-  if (!strcmp(param->get_name(), #reg)) { \
-    BX_CPU(cpu)->load_seg_reg(&BX_CPU(cpu)->sregs[BX_SEG_REG_##reg],val); \
-  }
-#define IF_LAZY_EFLAG_GET(flag) \
-    if (!strcmp(param->get_name(), #flag)) { \
-      return BX_CPU(cpu)->get_##flag(); \
-    }
-#define IF_LAZY_EFLAG_SET(flag, val) \
-    if (!strcmp(param->get_name(), #flag)) { \
-      BX_CPU(cpu)->set_##flag(val); \
-    }
-#define IF_EFLAG_GET(flag) \
-    if (!strcmp(param->get_name(), #flag)) { \
-      return BX_CPU(cpu)->get_##flag(); \
-    }
-#define IF_EFLAG_SET(flag, val) \
-    if (!strcmp(param->get_name(), #flag)) { \
-      BX_CPU(cpu)->set_##flag(val); \
-    }
-
-
-// implement get/set handler for parameters that need unusual set/get
-static Bit64s cpu_param_handler(bx_param_c *param, int set, Bit64s val)
-{
-#if BX_SUPPORT_SMP
-  int cpu = atoi(param->get_parent()->get_name());
-#endif
-  if (set) {
-    if (!strcmp(param->get_name(), "LDTR")) {
-      BX_CPU(cpu)->panic("setting LDTR not implemented");
-    }
-    if (!strcmp(param->get_name(), "TR")) {
-      BX_CPU(cpu)->panic("setting LDTR not implemented");
-    }
-    IF_SEG_REG_SET(CS, val);
-    IF_SEG_REG_SET(DS, val);
-    IF_SEG_REG_SET(SS, val);
-    IF_SEG_REG_SET(ES, val);
-    IF_SEG_REG_SET(FS, val);
-    IF_SEG_REG_SET(GS, val);
-    IF_LAZY_EFLAG_SET(OF, val);
-    IF_LAZY_EFLAG_SET(SF, val);
-    IF_LAZY_EFLAG_SET(ZF, val);
-    IF_LAZY_EFLAG_SET(AF, val);
-    IF_LAZY_EFLAG_SET(PF, val);
-    IF_LAZY_EFLAG_SET(CF, val);
-    IF_EFLAG_SET(ID,   val);
-    IF_EFLAG_SET(VIP,  val);
-    IF_EFLAG_SET(VIF,  val);
-    IF_EFLAG_SET(AC,   val);
-    IF_EFLAG_SET(VM,   val);
-    IF_EFLAG_SET(RF,   val);
-    IF_EFLAG_SET(NT,   val);
-    IF_EFLAG_SET(IOPL, val);
-    IF_EFLAG_SET(DF,   val);
-    IF_EFLAG_SET(IF,   val);
-    IF_EFLAG_SET(TF,   val);
-  } else {
-    if (!strcmp(param->get_name(), "LDTR")) {
-      return BX_CPU(cpu)->ldtr.selector.value;
-    }
-    if (!strcmp(param->get_name(), "TR")) {
-      return BX_CPU(cpu)->tr.selector.value;
-    }
-    IF_SEG_REG_GET (CS);
-    IF_SEG_REG_GET (DS);
-    IF_SEG_REG_GET (SS);
-    IF_SEG_REG_GET (ES);
-    IF_SEG_REG_GET (FS);
-    IF_SEG_REG_GET (GS);
-    IF_LAZY_EFLAG_GET(OF);
-    IF_LAZY_EFLAG_GET(SF);
-    IF_LAZY_EFLAG_GET(ZF);
-    IF_LAZY_EFLAG_GET(AF);
-    IF_LAZY_EFLAG_GET(PF);
-    IF_LAZY_EFLAG_GET(CF);
-    IF_EFLAG_GET(ID);
-    IF_EFLAG_GET(VIP);
-    IF_EFLAG_GET(VIF);
-    IF_EFLAG_GET(AC);
-    IF_EFLAG_GET(VM);
-    IF_EFLAG_GET(RF);
-    IF_EFLAG_GET(NT);
-    IF_EFLAG_GET(IOPL);
-    IF_EFLAG_GET(DF);
-    IF_EFLAG_GET(IF);
-    IF_EFLAG_GET(TF);
-  }
-  return val;
-}
-#undef IF_SEG_REG_GET
-#undef IF_SEG_REG_SET
-
-#endif
-
 #if BX_CPU_LEVEL >= 4
 
 #include "generic_cpuid.h"
@@ -187,8 +89,6 @@ static bx_cpuid_t *cpuid_factory(BX_CPU_C *cpu)
 // BX_CPU_C constructor
 void BX_CPU_C::initialize(void)
 {
-  BX_CPU_THIS_PTR set_INTR(0);
-
 #if BX_CPU_LEVEL >= 4
   BX_CPU_THIS_PTR cpuid = cpuid_factory(this);
   if (! BX_CPU_THIS_PTR cpuid)
@@ -224,137 +124,8 @@ void BX_CPU_C::initialize(void)
 #if BX_SUPPORT_VMX
   init_VMCS();
 #endif
-
-#if BX_WITH_WX
-  register_wx_state();
-#endif
 }
 
-#if BX_WITH_WX
-void BX_CPU_C::register_wx_state(void)
-{
-  if (SIM->get_param(BXPN_WX_CPU_STATE) != NULL) {
-    // Register some of the CPUs variables as shadow parameters so that
-    // they can be visible in the config interface.
-    // (Experimental, obviously not a complete list)
-    bx_param_num_c *param;
-    char cpu_name[10], cpu_title[10], cpu_pname[16];
-    const char *fmt16 = "%04X";
-    const char *fmt32 = "%08X";
-    Bit32u oldbase = bx_param_num_c::set_default_base(16);
-    const char *oldfmt = bx_param_num_c::set_default_format(fmt32);
-    sprintf(cpu_name, "%d", BX_CPU_ID);
-    sprintf(cpu_title, "CPU %d", BX_CPU_ID);
-    sprintf(cpu_pname, "%s.%d", BXPN_WX_CPU_STATE, BX_CPU_ID);
-    if (SIM->get_param(cpu_pname) == NULL) {
-      bx_list_c *list = new bx_list_c(SIM->get_param(BXPN_WX_CPU_STATE),
-         cpu_name, cpu_title);
-
-#define DEFPARAM_NORMAL(name,field) \
-    new bx_shadow_num_c(list, #name, &(field))
-
-      DEFPARAM_NORMAL(EAX, EAX);
-      DEFPARAM_NORMAL(EBX, EBX);
-      DEFPARAM_NORMAL(ECX, ECX);
-      DEFPARAM_NORMAL(EDX, EDX);
-      DEFPARAM_NORMAL(ESP, ESP);
-      DEFPARAM_NORMAL(EBP, EBP);
-      DEFPARAM_NORMAL(ESI, ESI);
-      DEFPARAM_NORMAL(EDI, EDI);
-      DEFPARAM_NORMAL(EIP, EIP);
-      DEFPARAM_NORMAL(DR0, dr[0]);
-      DEFPARAM_NORMAL(DR1, dr[1]);
-      DEFPARAM_NORMAL(DR2, dr[2]);
-      DEFPARAM_NORMAL(DR3, dr[3]);
-      DEFPARAM_NORMAL(DR6, dr6.val32);
-      DEFPARAM_NORMAL(DR7, dr7.val32);
-      DEFPARAM_NORMAL(CR0, cr0.val32);
-      DEFPARAM_NORMAL(CR2, cr2);
-      DEFPARAM_NORMAL(CR3, cr3);
-#if BX_CPU_LEVEL >= 5
-      DEFPARAM_NORMAL(CR4, cr4.val32);
-#endif
-
-      // segment registers require a handler function because they have
-      // special get/set requirements.
-#define DEFPARAM_SEG_REG(x) \
-    param = new bx_param_num_c(list, \
-      #x, #x, "", 0, 0xffff, 0); \
-    param->set_handler(cpu_param_handler); \
-    param->set_format(fmt16);
-#define DEFPARAM_GLOBAL_SEG_REG(name,field) \
-    param = new bx_shadow_num_c(list, \
-        #name"_base", &(field.base)); \
-    param = new bx_shadow_num_c(list, \
-        #name"_limit", &(field.limit));
-
-      DEFPARAM_SEG_REG(CS);
-      DEFPARAM_SEG_REG(DS);
-      DEFPARAM_SEG_REG(SS);
-      DEFPARAM_SEG_REG(ES);
-      DEFPARAM_SEG_REG(FS);
-      DEFPARAM_SEG_REG(GS);
-      DEFPARAM_SEG_REG(LDTR);
-      DEFPARAM_SEG_REG(TR);
-      DEFPARAM_GLOBAL_SEG_REG(GDTR, BX_CPU_THIS_PTR gdtr);
-      DEFPARAM_GLOBAL_SEG_REG(IDTR, BX_CPU_THIS_PTR idtr);
-#undef DEFPARAM_NORMAL
-#undef DEFPARAM_SEG_REG
-#undef DEFPARAM_GLOBAL_SEG_REG
-
-      param = new bx_shadow_num_c(list, "EFLAGS",
-          &BX_CPU_THIS_PTR eflags);
-
-      // flags implemented in lazy_flags.cc must be done with a handler
-      // that calls their get function, to force them to be computed.
-#define DEFPARAM_EFLAG(name) \
-    param = new bx_param_bool_c(list, \
-            #name, #name, "", get_##name()); \
-    param->set_handler(cpu_param_handler);
-#define DEFPARAM_LAZY_EFLAG(name) \
-    param = new bx_param_bool_c(list, \
-            #name, #name, "", get_##name()); \
-    param->set_handler(cpu_param_handler);
-
-#if BX_CPU_LEVEL >= 4
-      DEFPARAM_EFLAG(ID);
-      DEFPARAM_EFLAG(VIP);
-      DEFPARAM_EFLAG(VIF);
-      DEFPARAM_EFLAG(AC);
-#endif
-#if BX_CPU_LEVEL >= 3
-      DEFPARAM_EFLAG(VM);
-      DEFPARAM_EFLAG(RF);
-#endif
-#if BX_CPU_LEVEL >= 2
-      DEFPARAM_EFLAG(NT);
-      // IOPL is a special case because it is 2 bits wide.
-      param = new bx_shadow_num_c(
-              list,
-              "IOPL",
-              &BX_CPU_THIS_PTR eflags, 10,
-              12, 13);
-      param->set_range(0, 3);
-      param->set_format("%d");
-#endif
-      DEFPARAM_LAZY_EFLAG(OF);
-      DEFPARAM_EFLAG(DF);
-      DEFPARAM_EFLAG(IF);
-      DEFPARAM_EFLAG(TF);
-      DEFPARAM_LAZY_EFLAG(SF);
-      DEFPARAM_LAZY_EFLAG(ZF);
-      DEFPARAM_LAZY_EFLAG(AF);
-      DEFPARAM_LAZY_EFLAG(PF);
-      DEFPARAM_LAZY_EFLAG(CF);
-
-      // restore defaults
-      bx_param_num_c::set_default_base(oldbase);
-      bx_param_num_c::set_default_format(oldfmt);
-    }
-  }
-}
-#endif
-
 // save/restore functionality
 void BX_CPU_C::register_state(void)
 {
@@ -624,22 +395,15 @@ void BX_CPU_C::register_state(void)
   register_svm_state(cpu);
 #endif
 
+  BXRS_HEX_PARAM_SIMPLE32(cpu, pending_event);
+  BXRS_HEX_PARAM_SIMPLE32(cpu, event_mask);
   BXRS_HEX_PARAM_SIMPLE32(cpu, async_event);
-  BXRS_PARAM_BOOL(cpu, INTR, INTR);
 
 #if BX_X86_DEBUGGER
   BXRS_PARAM_BOOL(cpu, in_repeat, in_repeat);
-  // for debug only (no need for save/restore), calculated in prefetch()
-  BXRS_PARAM_BOOL(cpu, codebp, codebp);
 #endif
 
   BXRS_PARAM_BOOL(cpu, in_smm, in_smm);
-  BXRS_PARAM_BOOL(cpu, disable_SMI, disable_SMI);
-  BXRS_PARAM_BOOL(cpu, pending_SMI, pending_SMI);
-  BXRS_PARAM_BOOL(cpu, disable_NMI, disable_NMI);
-  BXRS_PARAM_BOOL(cpu, pending_NMI, pending_NMI);
-  BXRS_PARAM_BOOL(cpu, disable_INIT, disable_INIT);
-  BXRS_PARAM_BOOL(cpu, pending_INIT, pending_INIT);
 
 #if BX_DEBUGGER
   bx_list_c *tlb = new bx_list_c(cpu, "TLB");
@@ -942,15 +706,11 @@ void BX_CPU_C::reset(unsigned source)
 
 #if BX_X86_DEBUGGER
   BX_CPU_THIS_PTR in_repeat = 0;
-  BX_CPU_THIS_PTR codebp = 0;
 #endif
   BX_CPU_THIS_PTR in_smm = 0;
-  BX_CPU_THIS_PTR disable_SMI = 0;
-  BX_CPU_THIS_PTR pending_SMI = 0;
-  BX_CPU_THIS_PTR disable_NMI = 0;
-  BX_CPU_THIS_PTR pending_NMI = 0;
-  BX_CPU_THIS_PTR disable_INIT = 0;
-  BX_CPU_THIS_PTR pending_INIT = 0;
+
+  BX_CPU_THIS_PTR pending_event = 0;
+  BX_CPU_THIS_PTR event_mask = 0;
 
   if (source == BX_RESET_HARDWARE) {
     BX_CPU_THIS_PTR smbase = 0x30000; // do not change SMBASE on INIT
@@ -1051,7 +811,7 @@ void BX_CPU_C::reset(unsigned source)
       BX_CPU_THIS_PTR msr.mtrrfix4k[n] = 0;
 
     BX_CPU_THIS_PTR msr.pat = BX_CONST64(0x0007040600070406);
-    BX_CPU_THIS_PTR msr.mtrr_deftype = BX_CONST64(0x806);
+    BX_CPU_THIS_PTR msr.mtrr_deftype = 0;
   }
 #endif
 
@@ -1066,7 +826,7 @@ void BX_CPU_C::reset(unsigned source)
 #endif
 
   BX_CPU_THIS_PTR EXT = 0;
-  BX_CPU_THIS_PTR errorno = 0;
+  BX_CPU_THIS_PTR last_exception_type = 0;
 
   // invalidate the code prefetch queue
   BX_CPU_THIS_PTR eipPageBias = 0;
@@ -1121,10 +881,6 @@ void BX_CPU_C::reset(unsigned source)
 #if BX_SUPPORT_VMX
   BX_CPU_THIS_PTR in_vmx = BX_CPU_THIS_PTR in_vmx_guest = 0;
   BX_CPU_THIS_PTR in_smm_vmx = BX_CPU_THIS_PTR in_smm_vmx_guest = 0;
-  BX_CPU_THIS_PTR vmx_interrupt_window = 0;
-#if BX_SUPPORT_VMX >= 2  
-  BX_CPU_THIS_PTR pending_vmx_timer_expired = 0;
-#endif
   BX_CPU_THIS_PTR vmcsptr = BX_CPU_THIS_PTR vmxonptr = BX_INVALID_VMCSPTR;
   BX_CPU_THIS_PTR vmcshostptr = 0;
   /* enable VMX, should be done in BIOS instead */
@@ -1143,21 +899,23 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR in_event = 0;
 #endif
 
+#if BX_SUPPORT_VMX
+  BX_CPU_THIS_PTR nmi_unblocking_iret = 0;
+#endif
+
 #if BX_SUPPORT_SMP
   // notice if I'm the bootstrap processor.  If not, do the equivalent of
   // a HALT instruction.
   int apic_id = lapic.get_id();
   if (BX_BOOTSTRAP_PROCESSOR == apic_id) {
     // boot normally
-    BX_CPU_THIS_PTR msr.apicbase |= 0x0100; /* set bit 8 BSP */
+    BX_CPU_THIS_PTR msr.apicbase |=  0x100; /* set bit 8 BSP */
     BX_INFO(("CPU[%d] is the bootstrap processor", apic_id));
   } else {
     // it's an application processor, halt until IPI is heard.
-    BX_CPU_THIS_PTR msr.apicbase &= ~0x0100; /* clear bit 8 BSP */
-    BX_INFO(("CPU[%d] is an application processor. Halting until IPI.", apic_id));
-    activity_state = BX_ACTIVITY_STATE_WAIT_FOR_SIPI;
-    disable_INIT = 1; // INIT is disabled when CPU is waiting for SIPI
-    async_event = 1;
+    BX_CPU_THIS_PTR msr.apicbase &= ~0x100; /* clear bit 8 BSP */
+    BX_INFO(("CPU[%d] is an application processor. Halting until SIPI.", apic_id));
+    enter_sleep_state(BX_ACTIVITY_STATE_WAIT_FOR_SIPI);
   }
 #endif
 
