@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: msr.cc 11299 2012-07-26 16:03:26Z sshwarts $
+// $Id: msr.cc 11680 2013-04-17 19:59:56Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2008-2012 Stanislav Shwartsman
@@ -72,6 +72,10 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
 
 #if BX_CPU_LEVEL >= 6
     case BX_MSR_MTRRCAP:   // read only MSR
+      if (! bx_cpuid_support_mtrr()) {
+        BX_ERROR(("RDMSR MSR_MTRRCAP: MTRR is not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
       val64 = BX_CONST64(0x0000000000000508);
       break;
 
@@ -91,14 +95,26 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
     case BX_MSR_MTRRPHYSMASK6:
     case BX_MSR_MTRRPHYSBASE7:
     case BX_MSR_MTRRPHYSMASK7:
+      if (! bx_cpuid_support_mtrr()) {
+        BX_ERROR(("RDMSR: MTRR is not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
       val64 = BX_CPU_THIS_PTR msr.mtrrphys[index - BX_MSR_MTRRPHYSBASE0];
       break;
 
     case BX_MSR_MTRRFIX64K_00000:
+      if (! bx_cpuid_support_mtrr()) {
+        BX_ERROR(("RDMSR: MTRR is not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
       val64 = BX_CPU_THIS_PTR msr.mtrrfix64k_00000;
       break;
     case BX_MSR_MTRRFIX16K_80000:
     case BX_MSR_MTRRFIX16K_A0000:
+      if (! bx_cpuid_support_mtrr()) {
+        BX_ERROR(("RDMSR: MTRR is not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
       val64 = BX_CPU_THIS_PTR msr.mtrrfix16k[index - BX_MSR_MTRRFIX16K_80000];
       break;
 
@@ -110,15 +126,27 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
     case BX_MSR_MTRRFIX4K_E8000:
     case BX_MSR_MTRRFIX4K_F0000:
     case BX_MSR_MTRRFIX4K_F8000:
+      if (! bx_cpuid_support_mtrr()) {
+        BX_ERROR(("RDMSR: MTRR is not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
       val64 = BX_CPU_THIS_PTR msr.mtrrfix4k[index - BX_MSR_MTRRFIX4K_C0000];
       break;
 
-    case BX_MSR_PAT:
-      val64 = BX_CPU_THIS_PTR msr.pat;
+    case BX_MSR_MTRR_DEFTYPE:
+      if (! bx_cpuid_support_mtrr()) {
+        BX_ERROR(("RDMSR MSR_MTRR_DEFTYPE: MTRR is not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
+      val64 = BX_CPU_THIS_PTR msr.mtrr_deftype;
       break;
 
-    case BX_MSR_MTRR_DEFTYPE:
-      val64 = BX_CPU_THIS_PTR msr.mtrr_deftype;
+    case BX_MSR_PAT:
+      if (! bx_cpuid_support_pat()) {
+        BX_ERROR(("RDMSR BX_MSR_PAT: PAT is not enabled !"));
+        return handle_unknown_rdmsr(index, msr);
+      }
+      val64 = BX_CPU_THIS_PTR msr.pat;
       break;
 #endif
 
@@ -369,11 +397,16 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RDMSR(bxInstruction_c *i)
 #endif
 
 #if BX_SUPPORT_VMX >= 2
-  if (BX_CPU_THIS_PTR in_vmx_guest && index == 0x808) {
+  if (BX_CPU_THIS_PTR in_vmx_guest) {
     if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_VIRTUALIZE_X2APIC_MODE)) {
-      RAX = VMX_Read_VTPR() & 0xff;
-      RDX = 0;
-      BX_NEXT_INSTR(i);
+      if (index >= 0x800 && index <= 0x8FF) {
+        if (index == 0x808 || SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_VIRTUALIZE_APIC_REGISTERS)) {
+          unsigned vapic_offset = (index & 0xff) << 4;
+          RAX = VMX_Read_Virtual_APIC(vapic_offset);
+          RDX = VMX_Read_Virtual_APIC(vapic_offset + 4);
+          BX_NEXT_INSTR(i);
+        }
+      }
     }
   }
 #endif
@@ -397,9 +430,9 @@ bx_bool isMemTypeValidMTRR(unsigned memtype)
   case BX_MEMTYPE_WT:
   case BX_MEMTYPE_WP:
   case BX_MEMTYPE_WB:
-    return 1;
+    return BX_TRUE;
   default:
-    return 0;
+    return BX_FALSE;
   }
 }
 
@@ -417,9 +450,9 @@ bx_bool isValidMSR_PAT(Bit64u pat_msr)
       ! isMemTypeValidPAT((pat_msr >> 32) & 0xFF) ||
       ! isMemTypeValidPAT((pat_msr >> 40) & 0xFF) || 
       ! isMemTypeValidPAT((pat_msr >> 48) & 0xFF) || 
-      ! isMemTypeValidPAT(pat_msr >> 56)) return 0;
+      ! isMemTypeValidPAT(pat_msr >> 56)) return BX_FALSE;
 
-  return 1;
+  return BX_TRUE;
 }
 
 bx_bool isValidMSR_FixedMTRR(Bit64u fixed_mtrr_msr)
@@ -431,9 +464,9 @@ bx_bool isValidMSR_FixedMTRR(Bit64u fixed_mtrr_msr)
       ! isMemTypeValidMTRR((fixed_mtrr_msr >> 32) & 0xFF) ||
       ! isMemTypeValidMTRR((fixed_mtrr_msr >> 40) & 0xFF) || 
       ! isMemTypeValidMTRR((fixed_mtrr_msr >> 48) & 0xFF) || 
-      ! isMemTypeValidMTRR(fixed_mtrr_msr >> 56)) return 0;
+      ! isMemTypeValidMTRR(fixed_mtrr_msr >> 56)) return BX_FALSE;
 
-  return 1;
+  return BX_TRUE;
 }
 #endif
 
@@ -500,6 +533,10 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
 
 #if BX_CPU_LEVEL >= 6
     case BX_MSR_MTRRCAP:
+      if (! bx_cpuid_support_mtrr()) {
+        BX_ERROR(("WRMSR MSR_MTRRCAP: MTRR is not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
       BX_ERROR(("WRMSR: MTRRCAP is read only MSR"));
       return 0;
 
@@ -511,6 +548,10 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
     case BX_MSR_MTRRPHYSBASE5:
     case BX_MSR_MTRRPHYSBASE6:
     case BX_MSR_MTRRPHYSBASE7:
+      if (! bx_cpuid_support_mtrr()) {
+        BX_ERROR(("WRMSR: MTRR is not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
       if (! IsValidPhyAddr(val_64)) {
         BX_ERROR(("WRMSR[0x%08x]: attempt to write invalid phy addr to variable range MTRR %08x:%08x", index, val32_hi, val32_lo));
         return 0;
@@ -530,6 +571,10 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
     case BX_MSR_MTRRPHYSMASK5:
     case BX_MSR_MTRRPHYSMASK6:
     case BX_MSR_MTRRPHYSMASK7:
+      if (! bx_cpuid_support_mtrr()) {
+        BX_ERROR(("WRMSR: MTRR is not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
       if (! IsValidPhyAddr(val_64)) {
         BX_ERROR(("WRMSR[0x%08x]: attempt to write invalid phy addr to variable range MTRR %08x:%08x", index, val32_hi, val32_lo));
         return 0;
@@ -543,14 +588,23 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       break;
 
     case BX_MSR_MTRRFIX64K_00000:
+      if (! bx_cpuid_support_mtrr()) {
+        BX_ERROR(("WRMSR: MTRR is not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
       if (! isValidMSR_FixedMTRR(val_64)) {
         BX_ERROR(("WRMSR: attempt to write invalid Memory Type to MSR_MTRRFIX64K_00000 !"));
         return 0;
       }
       BX_CPU_THIS_PTR msr.mtrrfix64k_00000 = val_64;
       break;
+
     case BX_MSR_MTRRFIX16K_80000:
     case BX_MSR_MTRRFIX16K_A0000:
+      if (! bx_cpuid_support_mtrr()) {
+        BX_ERROR(("WRMSR: MTRR is not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
       if (! isValidMSR_FixedMTRR(val_64)) {
         BX_ERROR(("WRMSR: attempt to write invalid Memory Type to MSR_MTRRFIX16K regsiter !"));
         return 0;
@@ -566,6 +620,10 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
     case BX_MSR_MTRRFIX4K_E8000:
     case BX_MSR_MTRRFIX4K_F0000:
     case BX_MSR_MTRRFIX4K_F8000:
+      if (! bx_cpuid_support_mtrr()) {
+        BX_ERROR(("WRMSR: MTRR is not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
       if (! isValidMSR_FixedMTRR(val_64)) {
         BX_ERROR(("WRMSR: attempt to write invalid Memory Type to fixed memory range MTRR !"));
         return 0;
@@ -573,16 +631,11 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       BX_CPU_THIS_PTR msr.mtrrfix4k[index - BX_MSR_MTRRFIX4K_C0000] = val_64;
       break;
 
-    case BX_MSR_PAT:
-      if (! isValidMSR_PAT(val_64)) {
-        BX_ERROR(("WRMSR: attempt to write invalid Memory Type to MSR_PAT"));
-        return 0;
-      }
-      
-      BX_CPU_THIS_PTR msr.pat = val_64;
-      break;
-
     case BX_MSR_MTRR_DEFTYPE:
+      if (! bx_cpuid_support_mtrr()) {
+        BX_ERROR(("WRMSR MSR_MTRR_DEFTYPE: MTRR is not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
       if (! isMemTypeValidMTRR(val32_lo & 0xFF)) {
         BX_ERROR(("WRMSR: attempt to write invalid Memory Type to MSR_MTRR_DEFTYPE"));
         return 0;
@@ -593,6 +646,20 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       }
       BX_CPU_THIS_PTR msr.mtrr_deftype = val32_lo;
       break;
+
+    case BX_MSR_PAT:
+      if (! bx_cpuid_support_pat()) {
+        BX_ERROR(("WRMSR BX_MSR_PAT: PAT is not enabled !"));
+        return handle_unknown_wrmsr(index, val_64);
+      }
+
+      if (! isValidMSR_PAT(val_64)) {
+        BX_ERROR(("WRMSR: attempt to write invalid Memory Type to MSR_PAT"));
+        return 0;
+      }
+      
+      BX_CPU_THIS_PTR msr.pat = val_64;
+      break;
 #endif
 
     case BX_MSR_TSC:
@@ -659,7 +726,7 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
         BX_ERROR(("WRMSR SVM_HSAVE_PA_MSR: SVM support not enabled !"));
         return handle_unknown_wrmsr(index, val_64);
       }
-      if ((val_64 & 0xfff) != 0 || ! IsValidPhyAddr(val_64)) {
+      if (! IsValidPageAlignedPhyAddr(val_64)) {
         BX_ERROR(("WRMSR SVM_HSAVE_PA_MSR: invalid or not page aligned physical address !"));
       }
       BX_CPU_THIS_PTR msr.svm_hsave_pa = val_64;
@@ -672,7 +739,7 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
 
     case BX_MSR_STAR:
       if ((BX_CPU_THIS_PTR efer_suppmask & BX_EFER_SCE_MASK) == 0) {
-        BX_ERROR(("RDMSR MSR_STAR: SYSCALL/SYSRET support not enabled !"));
+        BX_ERROR(("WRMSR MSR_STAR: SYSCALL/SYSRET support not enabled !"));
         return handle_unknown_wrmsr(index, val_64);
       }
       MSR_STAR = val_64;
@@ -823,13 +890,16 @@ bx_bool BX_CPU_C::relocate_apic(Bit64u val_64)
     if (bx_cpuid_support_x2apic()) {
       unsigned apic_state = (BX_CPU_THIS_PTR msr.apicbase >> 10) & 3;
       unsigned new_state = (val32_lo >> 10) & 3;
-      if (new_state == BX_APIC_STATE_INVALID) {
-        BX_ERROR(("relocate_apic: attempt to set invalid apic state"));
-        return 0;
-      }
-      if (apic_state == BX_APIC_X2APIC_MODE && new_state != BX_APIC_GLOBALLY_DISABLED) {
-        BX_ERROR(("relocate_apic: attempt to switch from x2apic -> xapic"));
-        return 0;
+
+      if (new_state != apic_state) {
+        if (new_state == BX_APIC_STATE_INVALID) {
+          BX_ERROR(("relocate_apic: attempt to set invalid apic state"));
+          return 0;
+        }
+        if (apic_state == BX_APIC_X2APIC_MODE && new_state != BX_APIC_GLOBALLY_DISABLED) {
+          BX_ERROR(("relocate_apic: attempt to switch from x2apic -> xapic"));
+          return 0;
+        }
       }
     }
 #endif
@@ -856,6 +926,8 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::WRMSR(bxInstruction_c *i)
     exception(BX_GP_EXCEPTION, 0);
   }
 
+  invalidate_prefetch_q();
+
   Bit64u val_64 = ((Bit64u) EDX << 32) | EAX;
   Bit32u index = ECX;
 
@@ -871,10 +943,10 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::WRMSR(bxInstruction_c *i)
 #endif
 
 #if BX_SUPPORT_VMX >= 2
-  if (BX_CPU_THIS_PTR in_vmx_guest && index == 0x808) {
+  if (BX_CPU_THIS_PTR in_vmx_guest) {
     if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_VIRTUALIZE_X2APIC_MODE)) {
-      VMX_Write_VTPR(AL);
-      BX_NEXT_INSTR(i);
+      if (Virtualize_X2APIC_Write(index, val_64))
+        BX_NEXT_INSTR(i);
     }
   }
 #endif
@@ -883,7 +955,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::WRMSR(bxInstruction_c *i)
     exception(BX_GP_EXCEPTION, 0);
 #endif
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 #if BX_CONFIGURE_MSRS
