@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: proc_ctrl.cc 11313 2012-08-05 13:52:40Z sshwarts $
+// $Id: proc_ctrl.cc 11674 2013-04-09 15:43:15Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2013  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -50,7 +50,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PAUSE(bxInstruction_c *i)
 
 #if BX_SUPPORT_SVM
   if (BX_CPU_THIS_PTR in_svm_guest) {
-    if (SVM_INTERCEPT(SVM_INTERCEPT0_PAUSE)) Svm_Vmexit(SVM_VMEXIT_PAUSE);
+    if (SVM_INTERCEPT(SVM_INTERCEPT0_PAUSE)) SvmInterceptPAUSE();
   }
 #endif
 
@@ -72,7 +72,6 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::CPUID(bxInstruction_c *i)
 
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest) {
-    BX_DEBUG(("VMEXIT: CPUID in VMX non-root operation"));
     VMexit(VMX_VMEXIT_CPUID, 0);
   }
 #endif
@@ -110,16 +109,42 @@ void BX_CPU_C::shutdown(void)
   }
 #endif
 
-  BX_PANIC(("Entering to shutdown state still not implemented"));
+  enter_sleep_state(BX_ACTIVITY_STATE_SHUTDOWN);
 
-  BX_CPU_THIS_PTR clear_IF();
+  longjmp(BX_CPU_THIS_PTR jmp_buf_env, 1); // go back to main decode loop
+}
+
+void BX_CPU_C::enter_sleep_state(unsigned state)
+{
+  switch(state) {
+  case BX_ACTIVITY_STATE_ACTIVE:
+    BX_ASSERT(0); // should not be used for entering active CPU state
+    break;
+
+  case BX_ACTIVITY_STATE_HLT:
+    break;
+
+  case BX_ACTIVITY_STATE_WAIT_FOR_SIPI:
+    mask_event(BX_EVENT_INIT | BX_EVENT_SMI | BX_EVENT_NMI); // FIXME: all events should be masked
+    // fall through - mask interrupts as well
+
+  case BX_ACTIVITY_STATE_SHUTDOWN:
+    BX_CPU_THIS_PTR clear_IF(); // masking interrupts
+    break;
+   
+  case BX_ACTIVITY_STATE_MWAIT:
+  case BX_ACTIVITY_STATE_MWAIT_IF:
+    break;
+
+  default:
+    BX_PANIC(("enter_sleep_state: unknown state %d", state));
+  }
 
   // artificial trap bit, why use another variable.
-  BX_CPU_THIS_PTR activity_state = BX_ACTIVITY_STATE_HLT;
+  BX_CPU_THIS_PTR activity_state = state;
   BX_CPU_THIS_PTR async_event = 1; // so processor knows to check
-  // Execution of this instruction completes.  The processor
-  // will remain in a halt state until one of the above conditions
-  // is met.
+  // Execution completes.  The processor will remain in a sleep 
+  // state until one of the wakeup conditions is met.
 
   BX_INSTR_HLT(BX_CPU_ID);
 
@@ -130,8 +155,6 @@ void BX_CPU_C::shutdown(void)
 #if BX_USE_IDLE_HACK
   bx_gui->sim_is_idle();
 #endif
-
-  longjmp(BX_CPU_THIS_PTR jmp_buf_env, 1); // go back to main decode loop
 }
 
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::HLT(bxInstruction_c *i)
@@ -150,7 +173,6 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::HLT(bxInstruction_c *i)
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest) {
     if (VMEXIT(VMX_VM_EXEC_CTRL2_HLT_VMEXIT)) {
-      BX_DEBUG(("VMEXIT: HLT"));
       VMexit(VMX_VMEXIT_HLT, 0);
     }
   }
@@ -163,27 +185,11 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::HLT(bxInstruction_c *i)
 #endif
 
   // stops instruction execution and places the processor in a
-  // HALT state.  An enabled interrupt, NMI, or reset will resume
-  // execution.  If interrupt (including NMI) is used to resume
+  // HALT state. An enabled interrupt, NMI, or reset will resume
+  // execution. If interrupt (including NMI) is used to resume
   // execution after HLT, the saved CS:eIP points to instruction
   // following HLT.
-
-  // artificial trap bit, why use another variable.
-  BX_CPU_THIS_PTR activity_state = BX_ACTIVITY_STATE_HLT;
-  BX_CPU_THIS_PTR async_event = 1; // so processor knows to check
-  // Execution of this instruction completes.  The processor
-  // will remain in a halt state until one of the above conditions
-  // is met.
-
-  BX_INSTR_HLT(BX_CPU_ID);
-
-#if BX_DEBUGGER
-  bx_dbg_halt(BX_CPU_ID);
-#endif
-
-#if BX_USE_IDLE_HACK
-  bx_gui->sim_is_idle();
-#endif
+  enter_sleep_state(BX_ACTIVITY_STATE_HLT);
 
   BX_NEXT_TRACE(i);
 }
@@ -199,7 +205,6 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::INVD(bxInstruction_c *i)
 
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest) {
-    BX_ERROR(("VMEXIT: INVD in VMX non-root operation"));
     VMexit(VMX_VMEXIT_INVD, 0);
   }
 #endif
@@ -232,7 +237,6 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::WBINVD(bxInstruction_c *i)
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest) {
     if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_WBINVD_VMEXIT)) {
-      BX_ERROR(("VMEXIT: WBINVD in VMX non-root operation"));
       VMexit(VMX_VMEXIT_WBINVD, 0);
     }
   }
@@ -244,12 +248,12 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::WBINVD(bxInstruction_c *i)
   }
 #endif
 
-  invalidate_prefetch_q();
+//invalidate_prefetch_q();
 
-  BX_DEBUG(("WBINVD: Flush internal caches !"));
+  BX_DEBUG(("WBINVD: WB-Invalidate internal caches !"));
   BX_INSTR_CACHE_CNTRL(BX_CPU_ID, BX_INSTR_WBINVD);
 
-  flushICaches();
+//flushICaches();
 
   BX_NEXT_TRACE(i);
 }
@@ -287,7 +291,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::CLFLUSH(bxInstruction_c *i)
 #if BX_INSTRUMENTATION
   bx_phy_address paddr =
 #endif
-    translate_linear(laddr, USER_PL, BX_READ);
+    translate_linear(BX_TLB_ENTRY_OF(laddr), laddr, USER_PL, BX_READ);
 
   BX_INSTR_CLFLUSH(BX_CPU_ID, laddr, paddr);
 
@@ -451,10 +455,15 @@ void BX_CPU_C::handleCpuContextChange(void)
 
   invalidate_prefetch_q();
   invalidate_stack_cache();
+
+  handleInterruptMaskChange();
+
 #if BX_CPU_LEVEL >= 4
   handleAlignmentCheck();
 #endif
+
   handleCpuModeChange();
+
 #if BX_CPU_LEVEL >= 6
   handleSseModeChange();
 #if BX_SUPPORT_AVX
@@ -474,7 +483,6 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RDPMC(bxInstruction_c *i)
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest)  {
     if (VMEXIT(VMX_VM_EXEC_CTRL2_RDPMC_VMEXIT)) {
-      BX_DEBUG(("VMEXIT: RDPMC"));
       VMexit(VMX_VMEXIT_RDPMC, 0);
     }
   }
@@ -549,7 +557,6 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RDTSC(bxInstruction_c *i)
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest) {
     if (VMEXIT(VMX_VM_EXEC_CTRL2_RDTSC_VMEXIT)) {
-      BX_DEBUG(("VMEXIT: RDTSC"));
       VMexit(VMX_VMEXIT_RDTSC, 0);
     }
   }
@@ -594,7 +601,6 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RDTSCP(bxInstruction_c *i)
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest) {
     if (VMEXIT(VMX_VM_EXEC_CTRL2_RDTSC_VMEXIT)) {
-      BX_DEBUG(("VMEXIT: RDTSCP"));
       VMexit(VMX_VMEXIT_RDTSCP, 0);
     }
   }
@@ -658,7 +664,6 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MONITOR(bxInstruction_c *i)
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest) {
     if (VMEXIT(VMX_VM_EXEC_CTRL2_MONITOR_VMEXIT)) {
-      BX_DEBUG(("VMEXIT: MONITOR"));
       VMexit(VMX_VMEXIT_MONITOR, 0);
     }
   }
@@ -699,7 +704,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MONITOR(bxInstruction_c *i)
     }
   }
 
-  bx_phy_address paddr = translate_linear(laddr, USER_PL, BX_READ);
+  bx_phy_address paddr = translate_linear(BX_TLB_ENTRY_OF(laddr), laddr, USER_PL, BX_READ);
 
 #if BX_SUPPORT_SVM
   if (BX_CPU_THIS_PTR in_svm_guest) {
@@ -734,7 +739,6 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MWAIT(bxInstruction_c *i)
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest) {
     if (VMEXIT(VMX_VM_EXEC_CTRL2_MWAIT_VMEXIT)) {
-      BX_DEBUG(("VMEXIT: MWAIT"));
       VMexit(VMX_VMEXIT_MWAIT, BX_CPU_THIS_PTR monitor.armed);
     }
   }
@@ -774,35 +778,23 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MWAIT(bxInstruction_c *i)
   // the execution. Any far control transfer between MONITOR and MWAIT
   // resets the monitoring logic.
 
+  Bit32u new_state = BX_ACTIVITY_STATE_MWAIT;
   if (ECX & 1) {
 #if BX_SUPPORT_VMX
     // When "interrupt window exiting" VMX control is set MWAIT instruction
-    // won't cause the processor to enter BX_ACTIVITY_STATE_MWAIT_IF sleep
-    // state with EFLAGS.IF = 0
-    if (BX_CPU_THIS_PTR vmx_interrupt_window && ! BX_CPU_THIS_PTR get_IF()) {
-      BX_NEXT_TRACE(i);
+    // won't cause the processor to enter sleep state with EFLAGS.IF = 0
+    if (BX_CPU_THIS_PTR in_vmx_guest) {
+      if (VMEXIT(VMX_VM_EXEC_CTRL2_INTERRUPT_WINDOW_VMEXIT) && ! BX_CPU_THIS_PTR get_IF()) {
+        BX_NEXT_TRACE(i);
+      }
     }
 #endif
-    BX_CPU_THIS_PTR activity_state = BX_ACTIVITY_STATE_MWAIT_IF;
+    new_state = BX_ACTIVITY_STATE_MWAIT_IF;
   }
-  else {
-    BX_CPU_THIS_PTR activity_state = BX_ACTIVITY_STATE_MWAIT;
-  }
-
-  BX_CPU_THIS_PTR async_event = 1; // so processor knows to check
-  // Execution of this instruction completes.  The processor
-  // will remain in a optimized state until one of the above
-  // conditions is met.
 
   BX_INSTR_MWAIT(BX_CPU_ID, BX_CPU_THIS_PTR monitor.monitor_addr, CACHE_LINE_SIZE, ECX);
 
-#if BX_USE_IDLE_HACK
-  bx_gui->sim_is_idle();
-#endif
-
-#if BX_DEBUGGER
-  bx_dbg_halt(BX_CPU_ID);
-#endif
+  enter_sleep_state(new_state);
 #endif
 
   BX_NEXT_TRACE(i);
@@ -1065,8 +1057,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SYSCALL(bxInstruction_c *i)
     BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.l            = 0;
     BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.avl          = 0; /* available for use by system */
 
-    writeEFlags(read_eflags() & (~MSR_FMASK), EFlagsValidMask);
-    BX_CPU_THIS_PTR clear_RF();
+    writeEFlags(read_eflags() & ~MSR_FMASK & ~(EFlagsRFMask), EFlagsValidMask);
     RIP = temp_RIP;
   }
   else
@@ -1264,6 +1255,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SYSRET(bxInstruction_c *i)
 }
 
 #if BX_SUPPORT_X86_64
+
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SWAPGS(bxInstruction_c *i)
 {
   if(CPL != 0)
@@ -1351,4 +1343,5 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::WRGSBASE(bxInstruction_c *i)
 
   BX_NEXT_INSTR(i);
 }
+
 #endif
