@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: cpudb.h 11196 2012-05-26 19:40:18Z sshwarts $
+// $Id: cpudb.h 11679 2013-04-17 19:46:11Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2011-2012 Stanislav Shwartsman
+//   Copyright (c) 2011-2013 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -22,10 +22,10 @@
 /////////////////////////////////////////////////////////////////////////
 
 bx_define_cpudb(bx_generic)
-#if BX_CPU_LEVEL == 5
+#if BX_CPU_LEVEL >= 5
 bx_define_cpudb(pentium_mmx)
 bx_define_cpudb(amd_k6_2_chomper)
-#elif BX_CPU_LEVEL >= 6
+#if BX_CPU_LEVEL >= 6
 bx_define_cpudb(p2_klamath)
 bx_define_cpudb(p3_katmai)
 bx_define_cpudb(p4_willamette)
@@ -41,8 +41,10 @@ bx_define_cpudb(core2_penryn_t9600)
 bx_define_cpudb(corei5_lynnfield_750)
 bx_define_cpudb(corei5_arrandale_m520)
 #if BX_SUPPORT_AVX
+bx_define_cpudb(zambezi)
 bx_define_cpudb(corei7_sandy_bridge_2600k)
 bx_define_cpudb(corei7_ivy_bridge_3770k)
 #endif
 #endif
 #endif
+#endif
