@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: paramtree.h 11035 2012-02-14 18:13:54Z vruppert $
+// $Id: paramtree.h 11655 2013-03-17 17:16:45Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2010  The Bochs Project
+//  Copyright (C) 2010-2013  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -111,6 +111,11 @@ protected:
   bx_list_c *dependent_list;
   void *device;
 public:
+  enum {
+    // If set, this parameter is available in CI only. In bochsrc, it is set
+    // indirectly from one or more other options (e.g. cpu count)
+    CI_ONLY = (1<<31)
+  } bx_param_opt_bits;
   bx_param_c(Bit32u id, const char *name, const char *description);
   bx_param_c(Bit32u id, const char *name, const char *label, const char *description);
   virtual ~bx_param_c();
@@ -376,6 +381,8 @@ public:
   char get_separator() const {return separator; }
   int get_maxsize() const {return maxsize; }
   void set_initial_val(const char *buf);
+  bx_bool isempty();
+  int sprint(char *buf, int buflen, bx_bool dquotes);
 #if BX_USE_TEXTCONFIG
   virtual void text_print(FILE *fp);
   virtual int text_ask(FILE *fpin, FILE *fpout);
@@ -467,7 +474,12 @@ public:
     // When a bx_list_c is displayed as a menu, SHOW_GROUP_NAME controls whether
     // or not the name of group the item belongs to is added to the name of the
     // item (used in the runtime menu).
-    SHOW_GROUP_NAME = (1<<4)
+    SHOW_GROUP_NAME = (1<<4),
+    // When a bx_list_c is displayed in a dialog, USE_SCROLL_WINDOW suggests
+    // to the CI that the list items should be displayed in a scrollable dialog
+    // window. Large lists can make the dialog unusable and using this flag
+    // can force the CI to limit the dialog height with all items accessible.
+    USE_SCROLL_WINDOW = (1<<5)
   } bx_listopt_bits;
   bx_list_c(bx_param_c *parent);
   bx_list_c(bx_param_c *parent, const char *name);
