@@ -1,7 +1,9 @@
 /////////////////////////////////////////////////////////////////
-// $Id: wxdialog.cc 11339 2012-08-15 12:47:08Z vruppert $
+// $Id: wxdialog.cc 11633 2013-02-16 12:22:13Z vruppert $
 /////////////////////////////////////////////////////////////////
-
+//
+//  Copyright (C) 2002-2013  The Bochs Project
+//
 // Define BX_PLUGGABLE in files that can be compiled into plugins.  For
 // platforms that require a special tag on exported symbols, BX_PLUGGABLE
 // is used to know when we are exporting symbols and when we are importing.
@@ -305,7 +307,6 @@ void AdvancedLogOptionsDialog::Init()
 
 void AdvancedLogOptionsDialog::CopyParamToGui()
 {
-  SIM->apply_log_actions_by_device(); // settings from bochsrc
   bx_param_string_c *logfile = SIM->get_param_string(BXPN_LOG_FILENAME);
   SetLogfile(wxString(logfile->getptr(), wxConvUTF8));
   // copy log action settings from siminterface to gui
@@ -349,7 +350,7 @@ void AdvancedLogOptionsDialog::SetAction(int dev, int evtype, int act) {
     }
   }
   // this can happen if one of the choices that is excluded by
-  // ADVLOG_OPTS_EXCLUDE() is used, for example.
+  // BX_LOG_OPTS_EXCLUDE() is used, for example.
   wxLogDebug(wxT("warning: SetAction type=%d act=%d not found"), evtype, act);
 }
 
@@ -399,158 +400,6 @@ void AdvancedLogOptionsDialog::ShowHelp()
   wxMessageBox(MSG_NO_HELP, MSG_NO_HELP_CAPTION, wxOK | wxICON_ERROR, this);
 }
 
-#if BX_DEBUGGER
-//////////////////////////////////////////////////////////////////////
-// DebugLogDialog implementation
-//////////////////////////////////////////////////////////////////////
-// Structure:
-//   mainSizer:
-//     wxTextCtrl log (multiline with vert scrollbar)
-//     "Type a debugger command"
-//     commandSizer:
-//       wxTextCtrl command
-//       Execute button
-//     buttonSizer:
-//       help
-//       cancel
-//       ok
-//
-
-// all events go to OnEvent method
-BEGIN_EVENT_TABLE(DebugLogDialog, wxDialog)
-  EVT_BUTTON(-1, DebugLogDialog::OnEvent)
-  EVT_CHECKBOX(-1, DebugLogDialog::OnEvent)
-  EVT_KEY_DOWN(DebugLogDialog::OnKeyEvent)
-  EVT_KEY_UP(DebugLogDialog::OnKeyEvent)
-  EVT_CHAR(DebugLogDialog::OnKeyEvent)
-  EVT_TEXT(-1, DebugLogDialog::OnEvent)
-  EVT_TEXT_ENTER(-1, DebugLogDialog::OnEnterEvent)
-END_EVENT_TABLE()
-
-
-DebugLogDialog::DebugLogDialog(
-    wxWindow* parent,
-    wxWindowID id)
-  : wxDialog (parent, id, wxT(""), wxDefaultPosition, wxDefaultSize,
-    wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER)
-{
-  lengthMax = DEBUG_LOG_DEFAULT_LENGTH_MAX;
-  lengthTolerance = DEBUG_LOG_DEFAULT_TOLERANCE;
-  SetTitle (DEBUG_LOG_TITLE);
-  mainSizer = new wxBoxSizer (wxVERTICAL);
-  log = new wxTextCtrl (this, -1, wxT(""),
-      wxDefaultPosition, wxSize(400, 300),
-      wxTE_MULTILINE | wxTE_RICH | wxTE_READONLY);
-  mainSizer->Add (log, 1, wxALL|wxGROW, 10);
-  wxStaticText *text = new wxStaticText (this, -1, DEBUG_CMD_PROMPT);
-  mainSizer->Add (text, 0, wxTOP|wxLEFT, 10);
-  commandSizer = new wxBoxSizer (wxHORIZONTAL);
-  mainSizer->Add (commandSizer, 0, wxALL|wxGROW, 5);
-  buttonSizer = new wxBoxSizer (wxHORIZONTAL);
-  mainSizer->Add (buttonSizer, 0, wxALIGN_RIGHT);
-
-  // commandSizer contents
-  command = new wxTextCtrl (this, ID_DebugCommand, wxT(""),
-      wxDefaultPosition, wxDefaultSize,
-      wxTE_PROCESS_ENTER);
-  commandSizer->Add (command, 1, wxGROW);
-  wxButton *btn;
-  btn = new wxButton (this, ID_Execute, BTNLABEL_EXECUTE);
-  commandSizer->Add (btn, 0, wxALL, 5);
-
-  // buttonSizer contents
-  btn = new wxButton (this, wxID_OK, BTNLABEL_CLOSE);
-  buttonSizer->Add (btn, 0, wxALL, 5);
-}
-
-void DebugLogDialog::Init()
-{
-  // lay it out!
-  SetAutoLayout(TRUE);
-  SetSizer(mainSizer);
-  mainSizer->Fit(this);
-  wxSize size = mainSizer->GetMinSize();
-  int margin = 5;
-  SetSizeHints(size.GetWidth() + margin, size.GetHeight() + margin);
-  Center();
-}
-
-void DebugLogDialog::Execute(bool clear)
-{
-  // send to debugger
-  theFrame->DebugCommand (command->GetValue ());
-  // display what they typed on the log screen
-  if (clear) command->Clear ();
-}
-
-void DebugLogDialog::CheckLogLength ()
-{
-  // truncate the text control periodically to avoid a
-  // serious memory leak.
-  wxString str = log->GetValue ();
-  Bit32u len = str.Length ();
-  if (len > lengthMax + lengthTolerance) {
-    // Truncate the string.  Start from length - lengthMax, search
-    // forward until we find the first \n.
-    for (int i = len - lengthMax; i<(int)(len-1); i++) {
-      if (str.GetChar (i) == '\n') {
-	// remove the \n and everything before it.  Done.
-	//printf ("truncating from 0 to %d\n", i+1);
-	//printf ("\n");
-	log->Remove (0, i+1);
-	return;
-      }
-    }
-    // no newline found?!
-    //printf ("no newline found! truncating from 0 to %d", len - lengthMax);
-    //printf ("\n");
-    log->Remove (0, len - lengthMax);
-  } else {
-    //printf ("log length is %d, no truncation yet", len);
-    //printf ("\n");
-  }
-}
-
-void DebugLogDialog::AppendCommand (const char *cmd)
-{
-  log->AppendText(wxT(">>> "));
-  log->AppendText(wxString(cmd, wxConvUTF8));
-  log->AppendText(wxT("\n"));
-  int n = log->GetLastPosition();
-  if (n>0) n--;
-  log->ShowPosition(n);
-  CheckLogLength();
-}
-
-void DebugLogDialog::AppendText (wxString text) {
-  log->AppendText (text);
-  int n = log->GetLastPosition ();
-  if (n>0) n--;
-  log->ShowPosition (n);
-  CheckLogLength ();
-}
-
-void DebugLogDialog::OnEvent(wxCommandEvent& event)
-{
-  int id = event.GetId();
-  switch (id) {
-    case wxID_OK:
-      Show(FALSE);
-      break;
-    case ID_Execute: // pressed execute button
-      Execute(false);
-      break;
-    default:
-      event.Skip();
-  }
-}
-
-void DebugLogDialog::OnKeyEvent(wxKeyEvent& event)
-{
-  wxLogDebug(wxT("key event"));
-}
-#endif
-
 //////////////////////////////////////////////////////////////////////
 // PluginControlDialog implementation
 //////////////////////////////////////////////////////////////////////
@@ -885,45 +734,33 @@ void ParamDialog::AddParam (
         break;
       }
     case BXT_PARAM_STRING: {
-	bx_param_string_c *param = (bx_param_string_c*) param_generic;
-	if (!plain) ADD_LABEL (prompt);
+        bx_param_string_c *param = (bx_param_string_c*) param_generic;
+        char value[1024];
+        if (!plain) ADD_LABEL(prompt);
         bool isFilename = param->get_options() & param->IS_FILENAME;
-	wxTextCtrl *txtctrl = new wxTextCtrl (context->parent, pstr->id, wxT(""), wxDefaultPosition, isFilename? longTextSize : normalTextSize);
+        wxTextCtrl *txtctrl = new wxTextCtrl (context->parent, pstr->id, wxT(""), wxDefaultPosition, isFilename? longTextSize : normalTextSize);
         if (description) txtctrl->SetToolTip(wxString(description, wxConvUTF8));
-        if (param->get_options() & param->RAW_BYTES) {
-          char *value = param->getptr();
-          wxString buffer;
-          char sep_string[2];
-          sep_string[0] = param->get_separator();
-          sep_string[1] = 0;
-          for (int i=0; i<param->get_maxsize(); i++) {
-            wxString eachbyte;
-            eachbyte.Printf(wxT("%02x"), (unsigned int)0xff&value[i]);
-            if (i > 0)
-              buffer += wxString(sep_string, wxConvUTF8);
-            buffer += eachbyte;
-          }
-          txtctrl->SetValue(buffer);
-        } else {
-          txtctrl->SetValue(wxString(param->getptr(), wxConvUTF8));
+        param->sprint(value, 1024, 0);
+        txtctrl->SetValue(wxString(value, wxConvUTF8));
+        if ((param->get_options() & param->RAW_BYTES) == 0) {
           txtctrl->SetMaxLength(param->get_maxsize());
         }
-	sizer->Add (txtctrl, 0, wxALL, 2);
-	if (!plain) {
-	  if (isFilename) {
-	    // create Browse button
-	    pstr->browseButtonId = genId ();
-	    pstr->browseButton = new wxButton (context->parent,
-		pstr->browseButtonId, BTNLABEL_BROWSE);
-	    sizer->Add (pstr->browseButton, 0, wxALL, 2);
-	    idHash->Put (pstr->browseButtonId, pstr);  // register under button id
-	  } else {
-	    sizer->Add (1, 1);  // spacer
-	  }
-	}
-	pstr->u.text = txtctrl;
-	idHash->Put (pstr->id, pstr);
-	paramHash->Put (pstr->param->get_id (), pstr);
+        sizer->Add (txtctrl, 0, wxALL, 2);
+        if (!plain) {
+          if (isFilename) {
+            // create Browse button
+            pstr->browseButtonId = genId ();
+            pstr->browseButton = new wxButton (context->parent,
+              pstr->browseButtonId, BTNLABEL_BROWSE);
+            sizer->Add (pstr->browseButton, 0, wxALL, 2);
+            idHash->Put (pstr->browseButtonId, pstr);  // register under button id
+          } else {
+            sizer->Add (1, 1);  // spacer
+          }
+        }
+        pstr->u.text = txtctrl;
+        idHash->Put (pstr->id, pstr);
+        paramHash->Put (pstr->param->get_id (), pstr);
         break;
       }
     case BXT_LIST: {
@@ -1300,230 +1137,6 @@ void ParamDialog::ShowHelp()
   wxMessageBox(MSG_NO_HELP, MSG_NO_HELP_CAPTION, wxOK | wxICON_ERROR, this);
 }
 
-/////////////////////////////////////////////////////////////////
-// CpuRegistersDialog
-/////////////////////////////////////////////////////////////////
-// Structure:
-// - mainSizer
-//   - mainRegsSizer (grid or flexgrid)
-//     - col0: flexgrid
-//       - params from EAX to ESP
-//     - col1: flexgrid
-//       - params from EIP to EFLAGS
-//     - col2: flexgrid
-//       - params from LDTR to IDTR limit
-//   - flagsSizer
-//   - extRegsSizer
-//     - col0: flexgrid
-//       - DR* params
-//     - col1: flexgrid
-//       - TR* params
-//     - col2: flexgrid
-//       - CR* params
-
-// all events go to OnEvent method
-BEGIN_EVENT_TABLE(CpuRegistersDialog, wxDialog)
-  EVT_BUTTON(-1, CpuRegistersDialog::OnEvent)
-  EVT_CHECKBOX(-1, CpuRegistersDialog::OnEvent)
-  EVT_TEXT(-1, CpuRegistersDialog::OnEvent)
-END_EVENT_TABLE()
-
-CpuRegistersDialog::CpuRegistersDialog(
-    wxWindow* parent,
-    wxWindowID id)
-  : ParamDialog(parent, id)
-{
-  wxFlexGridSizer *column;
-  nflags = 0;
-  const char *mainRegList1[] = CPU_REGS_MAIN_REGS1;
-  const char *mainRegList2[] = CPU_REGS_MAIN_REGS2;
-  const char *mainRegList3[] = CPU_REGS_MAIN_REGS3;
-  const char *flagList[]     = CPU_REGS_FLAGS;
-  const char *controlList[]  = CPU_REGS_CONTROL_REGS;
-  const char *debugList[]    = CPU_REGS_DEBUG_REGS;
-  const char *testList[]     = CPU_REGS_TEST_REGS;
-  bx_list_c *base      = (bx_list_c*)SIM->get_param(BXPN_WX_CPU0_STATE);
-
-  // top level objects
-  wxStaticBox *mainRegsBox = new wxStaticBox(this, -1, wxT("Basic Registers"));
-  wxStaticBoxSizer *mainRegsBoxSizer =
-    new wxStaticBoxSizer(mainRegsBox, wxVERTICAL);
-  mainSizer->Add(mainRegsBoxSizer, 0, wxALL|wxGROW, 10);
-
-  wxStaticBox *flagsBox = new wxStaticBox(this, -1, wxT("EFLAGS Bits"));
-  wxStaticBoxSizer *flagsBoxSizer =
-    new wxStaticBoxSizer(flagsBox, wxVERTICAL);
-  mainSizer->Add(flagsBoxSizer, 0, wxALL|wxGROW, 10);
-
-  wxStaticBox *otherBox = new wxStaticBox(this, -1, wxT("Other Registers"));
-  wxStaticBoxSizer *otherBoxSizer =
-    new wxStaticBoxSizer(otherBox, wxVERTICAL);
-  mainSizer->Add(otherBoxSizer, 0, wxALL|wxGROW, 10);
-
-  // mainRegsSizer contents
-  mainRegsSizer = new wxFlexGridSizer(3);
-  mainRegsBoxSizer->Add(mainRegsSizer, 0, wxALL, 3);
-  column = new wxFlexGridSizer(3);
-  mainRegsSizer->Add(column, 0, wxALL, 10);
-  AddParamList(mainRegList1, base, column);
-
-  column = new wxFlexGridSizer(3);
-  mainRegsSizer->Add(column, 0, wxALL, 10);
-  AddParamList(mainRegList2, base, column);
-
-  column = new wxFlexGridSizer(3);
-  mainRegsSizer->Add(column, 0, wxALL, 10);
-  AddParamList(mainRegList3, base, column);
-
-  // add flag parameters
-  flagsSizer = new wxFlexGridSizer(CPU_REGS_MAX_FLAGS);
-  flagsBoxSizer->Add(flagsSizer, 0, wxALL | wxALIGN_CENTER, 3);
-  int i = 0;
-  while (flagList[i] != NULL) {
-    bx_param_c *param = SIM->get_param(flagList[i], base);
-    if (param != NULL) {
-      AddFlag(param);
-    }
-    i++;
-  }
-
-  // extRegsSizer contents
-  extRegsSizer = new wxFlexGridSizer(3);
-  otherBoxSizer->Add(extRegsSizer, 0, wxALL, 3);
-
-  column = new wxFlexGridSizer(3);
-  extRegsSizer->Add(column, 0, wxALL, 10);
-  AddParamList(controlList, base, column);
-
-  column = new wxFlexGridSizer(3);
-  extRegsSizer->Add(column, 0, wxALL, 10);
-  AddParamList(debugList, base, column);
-
-  column = new wxFlexGridSizer(3);
-  extRegsSizer->Add(column, 0, wxALL, 10);
-  AddParamList(testList, base, column);
-
-  // add buttons
-#if BX_DEBUGGER
-  // only show these if debugger is enabled
-  contButton = AddButton(ID_Debug_Continue, BTNLABEL_DEBUG_CONTINUE);
-  stopButton = AddButton(ID_Debug_Stop, BTNLABEL_DEBUG_STOP);
-  stepButton = AddButton(ID_Debug_Step, BTNLABEL_DEBUG_STEP);
-  //commitButton = AddButton(ID_Debug_Commit, BTNLABEL_DEBUG_COMMIT);
-#endif
-  AddButton(ID_Close, BTNLABEL_CLOSE);
-}
-
-void CpuRegistersDialog::AddFlag(bx_param_c *param)
-{
-  if (param == NULL) {
-    wxLogDebug(wxT("AddFlag on undefined param"));
-    return;
-  }
-  wxASSERT(nflags < CPU_REGS_MAX_FLAGS);
-  flagptr[nflags++] = param;
-}
-
-void CpuRegistersDialog::Init()
-{
-  int i;
-  for (i=0; i<CPU_REGS_MAX_FLAGS; i++) {
-    if (i<nflags) {
-      bx_param_c *param = flagptr[i];
-      flagsSizer->Add(new wxStaticText(this, -1, wxString(param->get_label(), wxConvUTF8)), 0, wxALL|wxALIGN_LEFT, 4);
-    } else {
-      flagsSizer->Add(0, 0);  // spacer
-    }
-  }
-  for (i=0; i<nflags; i++) {
-    bx_param_c *param = flagptr[i];
-    AddParam(param, flagsSizer, true);
-  }
-  // special case: make IOPL text field small
-  ParamStruct *pstr = (ParamStruct*)paramHash->Get(SIM->get_param(BXPN_WX_CPU0_EFLAGS_IOPL)->get_id());
-  if (pstr != NULL) {
-    wxSize size = pstr->u.text->GetSize();
-    size.SetWidth (size.GetWidth() / 2);
-    pstr->u.text->SetSize(size);
-    flagsSizer->SetItemMinSize(pstr->u.text, size.GetWidth(), size.GetHeight());
-  }
-  ParamDialog::Init();
-  stateChanged(false);
-}
-
-void CpuRegistersDialog::stateChanged (bool simRunning)
-{
-#if BX_DEBUGGER
-  contButton->Enable (!simRunning);
-  stepButton->Enable (!simRunning);
-  stopButton->Enable (simRunning);
-#endif
-}
-
-void CpuRegistersDialog::CopyParamToGui ()
-{
-  ParamDialog::CopyParamToGui ();
-#if BX_DEBUGGER
-  stateChanged (SIM->get_param_bool(BXPN_DEBUG_RUNNING)->get());
-#endif
-}
-
-// How am I going to communicate with the debugger?
-//
-// The current model is that the debugger asks you for a command, and
-// blocks forever until you press return.  Then it interprets the command,
-// however long it takes, and returns to the input loop when the command
-// is done.  A control-C can stop a command prematurely.
-//
-// To extend this into wxWidgets multithreaded space, I will create a
-// synchronous event called BX_SYNC_GET_DBG_COMMAND which is sent from
-// the simulation thread to wxWidgets.  When the user chooses a debugger
-// action (step, continue, breakpoint, etc.) the simulation awakens and
-// interprets the event by calling a function in bx_debug/dbg_main.cc.
-//
-// The equivalent of a control-C is pressing the "Stop" button during
-// a long step or continue command.  This can be implemented by setting
-// bx_guard.interrupt_requested = 1, just like the SIGINT handler in
-// bx_debug/dbg_main.cc does.
-//
-// input loop model is good.  Create a debugger input loop, possibly in
-// siminterface.
-// in the simulation thread.  This loop waits for a command from the
-// wxWidgets debugger
-//
-// For example, if you press the "Step" button 5
-// times, with each click it should call bx_dbg_stepN_command(1) in the
-// simulator thread.  When it returns, it goes back to
-//
-void CpuRegistersDialog::OnEvent(wxCommandEvent& event)
-{
-  int id = event.GetId ();
-  switch (id) {
-    case ID_Close:
-      Show(FALSE);
-      break;
-#if BX_DEBUGGER
-  case ID_Debug_Stop:
-      wxLogDebug(wxT("wxWidgets triggered a break"));
-      theFrame->DebugBreak();
-      break;
-    case ID_Debug_Continue:
-      wxLogDebug(wxT("before calling DebugCommand"));
-      theFrame->DebugCommand("continue");
-      wxLogDebug(wxT("after calling DebugCommand"));
-      break;
-    case ID_Debug_Step:
-      theFrame->DebugCommand("step 1");
-      break;
-    case ID_Debug_Commit:
-      CopyGuiToParam();
-      break;
-#endif
-    default:
-      ParamDialog::OnEvent(event);
-  }
-}
-
 //////////////////////////////////////////////////////////////////////
 // FloppyConfigDialog implementation
 //////////////////////////////////////////////////////////////////////
@@ -1578,7 +1191,7 @@ void FloppyConfigDialog::OnEvent(wxCommandEvent& event)
         createButton->Enable((val1 != BX_FDD_NONE) && (val2 != BX_FLOPPY_NONE));
       } else if ((pstr == pstrPath) && (!pstrPath->u.text->IsModified())) {
         pstrMedia->u.choice->SetSelection(pstrMedia->u.choice->FindString(wxT("auto")));
-        pstrStatus->u.checkbox->SetValue(1);
+        pstrStatus->u.choice->SetSelection(BX_INSERTED);
       }
     }
     ParamDialog::OnEvent(event);
@@ -1657,7 +1270,7 @@ void LogOptionsDialog::SetAction(int evtype, int a)
     }
   }
   // this can happen if one of the choices that is excluded by
-  // LOG_OPTS_EXCLUDE() is used, for example.
+  // BX_LOG_OPTS_EXCLUDE() is used, for example.
   wxLogDebug(wxT("SetAction type=%d a=%d not found"), evtype, a);
 }
 
@@ -1772,9 +1385,9 @@ wxChoice *makeLogOptionChoiceBox (wxWindow *parent,
   int lastChoice = 0;  // remember index of last choice
   int nchoice = includeNoChange? LOG_OPTS_N_CHOICES : LOG_OPTS_N_CHOICES_NORMAL;
   for (int choice=0; choice<nchoice; choice++) {
-    // the exclude expression allows some choices to not be available
-    // for some times.  For example, it would be stupid to ignore a panic.
-    if (!LOG_OPTS_EXCLUDE(evtype, choice)) {
+    // the exclude expression allows some choices not being available if they
+    // don't make any sense.  For example, it would be stupid to ignore a panic.
+    if (!BX_LOG_OPTS_EXCLUDE(evtype, choice)) {
       control->Append(choices[choice], &integers[choice]);
       // the client data is an int* that points to the choice number.
       // This is what will be returned by GetAction().
