@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: e1000.cc 11382 2012-08-30 20:41:25Z vruppert $
+// $Id: e1000.cc 11606 2013-02-01 19:13:58Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Intel(R) 82540EM Gigabit Ethernet support (ported from QEMU)
@@ -12,7 +12,7 @@
 //  Copyright (c) 2007 Dan Aloni
 //  Copyright (c) 2004 Antony T Curtis
 //
-//  Copyright (C) 2011  The Bochs Project
+//  Copyright (C) 2011-2013  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -319,12 +319,11 @@ Bit32s e1000_options_parser(const char *context, int num_params, char *params[])
     if (!SIM->get_param_bool("enabled", base)->get()) {
       if (valid == 0x04) {
         SIM->get_param_bool("enabled", base)->set(1);
-      } else if (valid < 0x80) {
-        BX_PANIC(("%s: 'e1000' directive incomplete (mac is required)", context));
       }
-    } else {
-      if (valid & 0x80) {
-        SIM->get_param_bool("enabled", base)->set(0);
+    }
+    if (valid < 0x80) {
+      if ((valid & 0x04) == 0) {
+        BX_PANIC(("%s: 'e1000' directive incomplete (mac is required)", context));
       }
     }
   } else {
@@ -335,8 +334,7 @@ Bit32s e1000_options_parser(const char *context, int num_params, char *params[])
 
 Bit32s e1000_options_save(FILE *fp)
 {
-  SIM->write_pci_nic_options(fp, (bx_list_c*) SIM->get_param(BXPN_E1000));
-  return 0;
+  return SIM->write_param_list(fp, (bx_list_c*) SIM->get_param(BXPN_E1000), NULL, 0);
 }
 
 // device plugin entry points
@@ -454,7 +452,7 @@ void bx_e1000_c::init(void)
     ((bx_param_bool_c*)((bx_list_c*)SIM->get_param(BXPN_PLUGIN_CTRL))->get_by_name("e1000"))->set(0);
     return;
   }
-  memcpy(macaddr, SIM->get_param_string("macaddr", base)->getptr(), 6);
+  memcpy(macaddr, SIM->get_param_string("mac", base)->getptr(), 6);
 
   memcpy(BX_E1000_THIS s.eeprom_data, e1000_eeprom_template,
          sizeof(e1000_eeprom_template));
@@ -479,7 +477,7 @@ void bx_e1000_c::init(void)
   BX_E1000_THIS pci_base_address[1] = 0;
   BX_E1000_THIS pci_rom_address = 0;
   bootrom = SIM->get_param_string("bootrom", base)->getptr();
-  if (strlen(bootrom) > 0) {
+  if ((strlen(bootrom) > 0) && (strcmp(bootrom, "none"))) {
     BX_E1000_THIS load_pci_rom(bootrom);
   }
 
