@@ -1,11 +1,11 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: es1370.cc 11384 2012-08-31 12:08:19Z vruppert $
+// $Id: es1370.cc 11595 2013-01-26 18:17:23Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 // ES1370 soundcard support (ported from QEMU)
 //
 // Copyright (c) 2005  Vassili Karpov (malc)
-// Copyright (C) 2011  The Bochs Project
+// Copyright (C) 2011-2013  The Bochs Project
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -123,11 +123,7 @@ Bit32s es1370_options_parser(const char *context, int num_params, char *params[]
   if (!strcmp(params[0], "es1370")) {
     bx_list_c *base = (bx_list_c*) SIM->get_param(BXPN_SOUND_ES1370);
     for (int i = 1; i < num_params; i++) {
-      if (!strncmp(params[i], "enabled=", 8)) {
-        SIM->get_param_bool("enabled", base)->set(atol(&params[i][8]));
-      } else if (!strncmp(params[i], "wavedev=", 8)) {
-        SIM->get_param_string("wavedev", base)->set(&params[i][8]);
-      } else {
+      if (SIM->parse_param_from_list(context, params[i], base) < 0) {
         BX_ERROR(("%s: unknown parameter for es1370 ignored.", context));
       }
     }
@@ -139,13 +135,7 @@ Bit32s es1370_options_parser(const char *context, int num_params, char *params[]
 
 Bit32s es1370_options_save(FILE *fp)
 {
-  bx_list_c *base = (bx_list_c*) SIM->get_param(BXPN_SOUND_ES1370);
-  fprintf(fp, "es1370: enabled=%d", SIM->get_param_bool("enabled", base)->get());
-  if (SIM->get_param_bool("enabled", base)->get()) {
-    fprintf(fp, ", wavedev=%s", SIM->get_param_string("wavedev", base)->getptr());
-  }
-  fprintf(fp, "\n");
-  return 0;
+  return SIM->write_param_list(fp, (bx_list_c*) SIM->get_param(BXPN_SOUND_ES1370), NULL, 0);
 }
 
 // device plugin entry points
@@ -214,7 +204,7 @@ void bx_es1370_c::init(void)
   }
   BX_ES1370_THIS pci_base_address[0] = 0;
 
-  char *wavedev = SIM->get_param_string(BXPN_ES1370_WAVEDEV)->getptr();
+  char *wavedev = SIM->get_param_string("wavedev", base)->getptr();
   if (!strcmp(wavedev, "sdl")) {
     BX_ES1370_THIS soundmod = DEV_sound_init_module("sdl", BX_ES1370_THIS_PTR);
   } else {
