@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: osdep.h 11371 2012-08-26 14:43:14Z vruppert $
+// $Id: osdep.h 11689 2013-05-24 17:58:49Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2009  The Bochs Project
+//  Copyright (C) 2001-2013  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -55,10 +55,12 @@ extern "C" {
 #define FMT_LL "%ll"
 #define FMT_TICK "%011llu"
 #define FMT_ADDRX64 "%016llx"
+#define FMT_PHY_ADDRX64 "%012llx"
 #else
 #define FMT_LL "%I64"
 #define FMT_TICK "%011I64u"
 #define FMT_ADDRX64 "%016I64x"
+#define FMT_PHY_ADDRX64 "%012I64x"
 #endif
 
 // always return regular file.
@@ -103,6 +105,7 @@ extern "C" {
 #define FMT_LL "%I64"
 #define FMT_TICK "%011I64u"
 #define FMT_ADDRX64 "%016I64x"
+#define FMT_PHY_ADDRX64 "%012I64x"
 
 #define off_t __int64
 // mingw gcc 4.6.1 already has lseek defined
@@ -116,10 +119,12 @@ extern "C" {
 #define FMT_LL "%l"
 #define FMT_TICK "%011lu"
 #define FMT_ADDRX64 "%016lx"
+#define FMT_PHY_ADDRX64 "%012lx"
 #else
 #define FMT_LL "%ll"
 #define FMT_TICK "%011llu"
 #define FMT_ADDRX64 "%016llx"
+#define FMT_PHY_ADDRX64 "%012llx"
 #endif
 #endif   /* not WIN32 definitions */
 
@@ -218,6 +223,109 @@ FILE *  fdopen(int fd, const char *type);
 typedef long ssize_t ;
 #endif
 
+//////////////////////////////////////////////////////////////////////
+// Missing library functions and byte-swapping stuff,
+// implemented for MorphOS only
+//////////////////////////////////////////////////////////////////////
+
+#ifdef __MORPHOS__
+int fseeko(FILE *stream, off_t offset, int whence);
+struct tm *localtime_r(const time_t *timep, struct tm *result);
+
+BX_CPP_INLINE Bit16u bx_ppc_bswap16(Bit16u val)
+{
+  Bit32u res;
+
+  __asm__("rlwimi %0,%0,16,8,15"
+          : "=r" (res)
+          : "0" (val));
+
+  return (Bit16u)(res >> 8);
+}
+
+BX_CPP_INLINE Bit32u bx_ppc_bswap32(Bit32u val)
+{
+  Bit32u res;
+
+  __asm__("rotlwi %0,%1,8\n\t"
+          "rlwimi %0,%1,24,0,7\n\t"
+          "rlwimi %0,%1,24,16,23"
+          : "=&r" (res)
+          : "r" (val));
+
+  return res;
+}
+
+BX_CPP_INLINE Bit64u bx_ppc_bswap64(Bit64u val)
+{
+  Bit32u hi, lo;
+
+  __asm__("rotlwi %0,%2,8\n\t"
+          "rlwimi %0,%2,24,0,7\n\t"
+          "rlwimi %0,%2,24,16,23\n\t"
+          "rotlwi %1,%3,8\n\t"
+          "rlwimi %1,%3,24,0,7\n\t"
+          "rlwimi %1,%3,24,16,23"
+          : "=&r" (hi), "=&r" (lo)
+          : "r" ((Bit32u)(val & 0xffffffff)), "r" ((Bit32u)(val >> 32)));
+
+  return ((Bit64u)hi << 32) | (Bit64u)lo;
+}
+
+BX_CPP_INLINE Bit16u bx_ppc_load_le16(const Bit16u *p)
+{
+  Bit16u v;
+  __asm__("lhbrx %0, 0, %1"
+          : "=r" (v)
+          : "r" (p), "m" (*p));
+  return v;
+}
+
+BX_CPP_INLINE void bx_ppc_store_le16(Bit16u *p, Bit16u v)
+{
+  __asm__("sthbrx %1, 0, %2"
+          : "=m" (*p)
+          : "r" (v), "r" (p));
+}
+
+BX_CPP_INLINE Bit32u bx_ppc_load_le32(const Bit32u *p)
+{
+  Bit32u v;
+  __asm__("lwbrx %0, 0, %1"
+          : "=r" (v)
+          : "r" (p), "m" (*p));
+  return v;
+}
+
+BX_CPP_INLINE void bx_ppc_store_le32(Bit32u *p, Bit32u v)
+{
+  __asm__("stwbrx %1, 0, %2"
+          : "=m" (*p)
+          : "r" (v), "r" (p));
+}
+
+BX_CPP_INLINE Bit64u bx_ppc_load_le64(const Bit64u *p)
+{
+  Bit32u hi, lo;
+
+  __asm__("lwbrx %0, 0, %2\n\t"
+          "lwbrx %1, 0, %3"
+          : "=&r" (lo), "=&r" (hi)
+          : "r" ((Bit32u *)p), "r" ((Bit32u *)p+1));
+
+  return ((Bit64u)hi << 32) | (Bit64u)lo;
+}
+
+BX_CPP_INLINE void bx_ppc_store_le64(Bit64u *p, Bit64u v)
+{
+  __asm__("stwbrx %1, 0, %3\n\t"
+          "stwbrx %2, 0, %4"
+          : "=m" (*p)
+          : "r" ((Bit32u)(v & 0xffffffff)), "r" ((Bit32u)(v >> 32)),
+            "r" ((Bit32u *)p), "r" ((Bit32u *)p+1));
+}
+#endif
+
 //////////////////////////////////////////////////////////////////////
 // New functions to replace library functions
 //   with OS-independent versions
